/*****
  This program creates a radio station ID reminder using the DS1307 RTC. The LCD can also serve as a 
  simple station clock. By default, however, it starts in the ID Timer mMde. The Timer Mode is indicated
  by an asterisk that bounces between rows 1 and 2 in column 15. After 10 minutes have passed, the LCD
  display begins to blink and continues to do so until the puch button is pressed for at least one second.
  After one second, the display stops blinking, and another 10 minute period begins.You can disable the
  Timer Mode by holding the push button for a period of more than 5 seconds, at which time the asterisks
  disappear from the display.
  
  Dec. 20, 2013
  Dr. Purdum, W9NMT/8
	 
*/
#include <Wire.h>            //I2C header file
#include <LiquidCrystal.h>
#include <RTClib.h>

#define DEBUG 1            // Use this to toggle debug stuff

#define RTCI2CADDRESS 0x68   // 7 bit device address (without last bit - look at the datasheet)
#define LEDPIN        13
#define RESETSWITCHPIN 8

#define IDTIMESLICE      600000  // 10 minutes. Change to shorter period for debugging...
#define IDTIMESLICE      10000    // For debugging, comment one above out...
#define HALFSECOND       500
#define DEBOUNCEDELAY    20
#define ERRORREPEAT      6       // The number of times to repeat blinking error LED indicator
#define FLIPDELAY        1       // Alternates LCD asterisk after this many seconds
#define MODESWITCHDELAY  5000    // If they hold the push botton longer than 5 seconds, activeTimer
                                 // changes state.
                                 
#define MILLENIA         2000

#define SECONDS          0      // These are offsets into the DS1307 internal registers
#define MINUTES          1
#define HOURS            2
#define DAY              3      // This is a day-of-the-week index register
#define DATE             4
#define MONTH            5
#define YEAR             6

#define RTCWRITEERROR 1 // Unable to write RTC time and date to DS1307
#define RTCREADERROR  2 // Unable to read RTC time and date from DS1307


LiquidCrystal lcd(12, 11, 5, 4, 3, 2); // For LCD display from Chapter 3
RTC_Millis rtc;

byte result;
byte timeBuffer[8];
byte flagBit = 0;

boolean activeTimer;        // Holds state of timer: 1 = active, 0 = inactive

char* weekdayname[] = {"", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};
char *timePeriods[10];

long millisStart, millisCurrent;

void setup()
{
#ifdef DEBUG
  Serial.begin(115200);
#endif
  
  Wire.begin();                           // Initiate the Wire library and join the I2C 
                                          // bus as a master
  lcd.begin(16, 2);                       // set up the LCD's number of columns and rows 

  pinMode(LEDPIN, OUTPUT);                // Set error LED
  pinMode(RESETSWITCHPIN, INPUT_PULLUP);  // The reset clock pushbutton
  
  millisStart = millis();                 // Start the count...
  activeTimer = true;                     // Assume they want timer started
}

void loop()
{  
  static boolean timesExpired = false;
  boolean featureChanged;
  byte sentBack;
  int i;
  static long switchMode;                // This tracks how long the button has been pushed.
  static long count = 0L;
  long timeThatSwitchWasClosed = 0L;
  
  millisCurrent = millis();
  if (activeTimer == true) {            // If they want to use the timer feature and
    if (millisCurrent - millisStart > IDTIMESLICE) {  // Is it time to tell them?
      timesExpired = true;                            // Yep...
    } 
    if (timesExpired) {             // Show them they're passed due for an ID
      lcd.noDisplay();                  
      delay(HALFSECOND);
      lcd.display();
      delay(HALFSECOND);
      DisplayDateAndTime(timeBuffer);
    }
  }
  if (debounce(RESETSWITCHPIN) == true) {  // They have pressed the switch? Yes, so...
    if (activeTimer == true) {
      timesExpired = false;                  // ...at least they want to reset timer...
      millisStart = millis();
    }
    switchMode = millis();    // Start measuring how long switch closed...
    count = millis();    // Start measuring how long switch closed...
    featureChanged = false;                         // Assume no change in timer feature...
    while (digitalRead(RESETSWITCHPIN) == LOW) {    // As long as it's LOW...
      count = millis();                            // Yep. Count how long it's LOW...
      timeThatSwitchWasClosed = count - switchMode;
      if (timeThatSwitchWasClosed >= MODESWITCHDELAY) { // Hold it long enough to toggle mode?
        featureChanged = true;                          // Yep, they did...
        break;
      }
     }          // End while() switch was closed
  }            // The switch is now opened

  if (featureChanged == true) {       // They want to change clock mode
    if (activeTimer == true) {        // So, if ID timer was running...
      lcd.setCursor(15, 0);           // Clear out asterisks
      lcd.print(' ');
      lcd.setCursor(15, 1);
      lcd.print(' ');
      activeTimer = false;            // ...shut the ID timer feature off         
    } else {                          // Otherwise, they must have wanted to turn it on.
      activeTimer = true;             // ...turn on the ID timer feature.
      millisStart = millis();         // Start the count
    }
    featureChanged = false;
    timesExpired = false;                  // ...at least they want to reset timer...
  }  
  
  Wire.beginTransmission(RTCI2CADDRESS);  // Send out the RTC device address on the bus
  Wire.write(0);                          // Reset internal register pointer             
  result = Wire.endTransmission();
  if (result)                             // Non-zero means we could write to the chip
    SetError(RTCWRITEERROR);              
 
  sentBack = Wire.requestFrom(RTCI2CADDRESS, 7);  // Ask for all 7 data bytes
  if (sentBack == 0)                              // It didn't like that request...
    SetError(RTCREADERROR);
    
  for (i = 0; i < sentBack; i++) {        // Read data, convert from BCD to decimal, and ...
    timeBuffer[i] = BcdToDec(Wire.read());// ...stuff into buffer
  }
  DisplayDateAndTime(timeBuffer);         // Show what we got back...
}



/*****
  This function is a pretty standard debounce function so we can determine that the reset switch
  has really been pressed.
  
  Parameter list:
    int pin            the pin that registers the switch state
    
  Return value:
    boolean            true if the switch was pressed, false otherwise
    
*****/
boolean debounce(int pin)
{
  boolean currentState;
  boolean previousState;
  int i; 
  
  previousState = digitalRead(pin);
  for (i = 0; i < DEBOUNCEDELAY; i++) {
    delay(1);    // small delay
    currentState = digitalRead(pin);      // Read it again...
    if (currentState != previousState)    // If not the same, it could be bouncing...
    {
      i = 0;                              // ...so try another pass...
      previousState = currentState;
    }
  }
  if (currentState == LOW)
    return true;
  else
    return false;
}

/*****
  This function displays the date and time returned from the RTC on the LCD display.
  
  Parameter list:
    void
    
  Return value:
    void
    
  CAUTION: This function assumes that all of the date-time variables have been read from
  the RTC before this function is called.
*****/
void DisplayDateAndTime(byte *ptr)
{  
#ifdef DEBUG
Serial.print("ptr = ");
Serial.print(*ptr);
Serial.print("ptr + DAY");
Serial.println(*(ptr + DAY));
#endif

    lcd.setCursor(0, 0);                  // First LCD display line
 //   lcd.print(weekdayname[*(ptr + DAY)]); // Note that you can use a pointer with an offset, or...
    lcd.print(weekdayname[ptr[DAY]]); // Note that you can use a pointer with an offset, or...
    lcd.print(" ");
    DigitFill(ptr[MONTH]);                // ...you can use it as an array index
    lcd.print("/");
    DigitFill(ptr[DATE]);
    lcd.print("/");
    DigitFill(ptr[YEAR]);
    
    lcd.setCursor(4, 1);                  // Second LCD display line
    DigitFill(ptr[HOURS]);
    lcd.print(":");
    DigitFill(ptr[MINUTES]);
    lcd.print(":");
    DigitFill(ptr[SECONDS]);
   
    if (activeTimer) {
      TimerIndicator(ptr);    // Display asterisk at line end if timing.
    }
 }

/*****
  This function determines if the value passed in requires one or two digits to display. 
  If the number is less than 10, the display is padded with a leading (tens units) of
  '0' and the units digit is displayed. Otherwise, the tens and units digits are displayed.
 
  Parameter list:
    byte digit        A number
    
  Return value:
    void
    
*****/  
void DigitFill(byte digit)
{
  if (digit < 10)
    lcd.print('0');
  lcd.print(digit);
}

/*****
  This function is used to indicate whether the ID timer is active or not. When the timer
  is not active, the LCD acts like a simple clock. When the timer is active, the LCD
  display blinks at the end of the ID period. This function places an asterisk in 
  column 15 of the display only when the timer is active. The asterisk alternates between
  rows 0 and 1 every FLIPDELAY seconds.
 
  Parameter list:
    byte value        A number expressed in BCD
    
  Return value:
    byte              The BCD value expressed as a decimal number
    
*****/ 
void TimerIndicator(byte *ptr) 
{
  static byte flip = 0;
  static byte oldSeconds, newSeconds;

  if (activeTimer == false)
    return;
  newSeconds = ptr[SECONDS];
  if (newSeconds == 0) {    // rollover at 0 seconds, leaving oldSeconds at 59, which would
    oldSeconds = 0;         // stop the display of the asterisks from then on, so reset it.
  }

  if (newSeconds - oldSeconds >= FLIPDELAY) {     // Flip once every FLIPDELAY seconds
    if (flip) {
      lcd.setCursor(15, 0);
      lcd.print("*");
      lcd.setCursor(15, 1);
      lcd.print(" ");
      flip = 0;
    } else {
      lcd.setCursor(15, 1);
      lcd.print("*");
      lcd.setCursor(15, 0);
      lcd.print(" ");
      flip = 1;
    }
    oldSeconds = newSeconds;
  }
}
/*****
  This function converts the byte-length Binary Coded Decimal value returned from the RTC
  to a decimal number.
 
  Parameter list:
    byte value        A number expressed in BCD
    
  Return value:
    byte              The BCD value expressed as a decimal number
    
*****/
byte BcdToDec(byte value)
{
  return ((value / 16) * 10 + value % 16);  // Think about it...
}

/*****
  This function converts the byte-length decimal value to a Binary Coded Decimal value. 
 
  Parameter list:
    byte value        A number expressed as a decimal value
    
  Return value:
    byte              The decimal value expressed as a BCD number
    
*****/
byte DecToBcd(byte value){
  return (value / 10 * 16 + value % 10);  // Think about this one, too.
}

/*****
  This method blinks the Arduino board LED if an error occurs. The error type is coded into the 
  blink rate, as explained below. There is a 10 second pause between the error blick rate. Note
  that you could assign LEDPIN to a different pin and run the LED "outside" of the Arduino board 
  to make it more visible.
 
  Parameter list:
     int             an integer that defines the type of error. These error values are #defines
                     for: RTCREADERROR = 2 and means we could not read the RTC chip, and
                     RTCWRITEERROR = 1 which means we could not write to the chip.
 
  Return value:
    void
*****/
void SetError(int error)
{
  int i;
  int repeat = ERRORREPEAT;     // The number of times to repeat the error indicator
  
  while(repeat--)               // Go back to the caller after this many repeats of 
  {                             // blinking the error LED for 10 seconds
    for (i = 0; i < error; i++)
    {
      digitalWrite(LEDPIN, HIGH);
      delay(500);
      digitalWrite(LEDPIN, LOW);
      delay(500);
    }
    delay(10000);              // wait 10 seconds and then repeat up to ERRORREPEAT times.
  }
}

/*****
  This method reads the high bit of the Seconds register of the DS1307 to see if the clock is 
  running. 
  
  Parameter list:
    void
    
  Return value:
    byte            1 if the RTC clock is running, 0 if not.
  
  CAUTION: This function is currently not used, but is present to show you one way to read a bit 
           value from a register if you so desire (e.g., change from 24 to 12 hour format and 
           implement AM/PM indicator.)
*****/
byte Isrunning(void)
{
  byte sentBack;
  uint8_t secondsRegister;
  
  Wire.beginTransmission(RTCI2CADDRESS);   // Wake up
  Wire.write(0);                           // Make sure RTC register pointer reset to 0
  Wire.endTransmission();

  sentBack = Wire.requestFrom(RTCI2CADDRESS, 1);      // Read Seconds register byte
  if (sentBack == 0)                                  // It didn't like that request...
    SetError(RTCREADERROR);

  secondsRegister = Wire.read();
  return !(secondsRegister >> 7);          // Get Clock Halt (CH) bit, invert it, and return it        
}
