/* Simple LCD Panel Meter ver4.1 4 January 2014 D.Kidder W6DQ

Features:
  * Along with associated hardware, displays 0-1 mA fullscale
  * Displays overrange condition when input exceeds 1 mA
  * Displays an analog bar graph on the second line of LCD
  
  The meter is user-configurable by changing the scaling value and scale.
  As presented, the associated hardware produces a 0-3.5 VDC output to the
  analog input of the Arduino.  The associated hardware also provides a 
  3.5VDC  source to the AREF input (analog reference), thus at 1 mA input, 
  the A/D count as 1000.  The hardware includes over and under-voltage 
  protection.  The meter hardware presents a 51 Ohm input impedance.
  
  The user may change the scale by changing the value of the "scale" string.  The 
  meter range may be modified by changing the "scaleFactor" variable.  Voltage 
  dividers or current shunts may be added to change the hardware range.

*/

#include <LiquidCrystal.h>  // Standard Arduino LCD library

/*
 * LCD Analog Bar Graph library (with modifications)
 *
 * Author: Balazs Kelemen
 * Contact: prampec+arduino@gmail.com
 * Copyright: 2010 Balazs Kelemen
 * Use is covered under the GNU GPL
 * This library creates an analog bar graph on the LCD
*/
#include <LcdBarGraph.h>

#define DEBUG 1

#define DEFAULTPRECISION 3	  // default decimal places
#define MAXANALOGVALUE   1000     // Set valid input to 0 - 1000
#define VALUEWIDTH       8       // Numeric field width, include '-', '.'

#define UPDATEDELAY      100      // Delay for normal reading
#define ERRORDELAY       400      // Delay if error message displayed
#define STARTUPDELAY     3000

#define BARGRAPHROW      1        // Draw graph on second row
#define BARGRAPHCOL      0        // Start graph in column 0

byte  lcdNumCols = 16;         // number of columns on LCD
byte  sensorPin = 1;           // define the analog input as A1

/*
*  The user can modify the following two lines to change scale and scale factor.
*
*  “scale[]” provides the displayed units of measurement. Replace the string within *  the quotes to assign the units you require.  E.g: “Volts” “Amperes” etc. 
*
*  “scaleFactor” is used to set the maximum displayed range of the measured value.
*  The default value of 1000.0 equates 1 mA input current (a count of 1000 in
*  the ADC) to 1.00 mA displayed. For example, if you wish to display 10 Volts
*  full scale, then 1 mA input current would be an ADC count of 1000 and the scale
*  factor would be 100.  The displayed range would be 0. to 10.00.  
*
*/

char  scale[] = "mA";         // scale string for display units
float scaleFactor = 1000.0;               // scaling factor for the meter 

LiquidCrystal lcd(12, 11, 5, 4, 3, 2); 	  // setup the LCD hardware interface pins
LcdBarGraph   lbg(&lcd, lcdNumCols, BARGRAPHCOL, BARGRAPHROW);

void setup()
{
  analogReference(EXTERNAL);      // set the A/D to use the AREF input

  lcd.begin(2, lcdNumCols);       // initialize the LCD

  lcd.print("LCD Panel Meter ");  //sappy stuff, name, version and date
  lcd.setCursor(0,1);
  lcd.print("ver 0.3 10DEC 13");
  delay(STARTUPDELAY);
  lcd.clear();

}

void loop()
{
  int i;
  int analogIn;
  int len;
  char buffer[lcdNumCols + 1];  // Make it big enough for a 4x20 display
  float val;
  /* Two states exist for the display -- in range and out of range
    (overrange).  Valid range is given as 0-1000 ADC count value.  
    The author made a decision that any count over 1000 is considered
    "overrange" and the second while statement handles this condition.
    The associated hardware includes a calibration 10-turn pot that is
    used to set 1.000 mA input current to a count of 1000, displaying 
    1.000 mA. Since the analog input values range from 0-1023, the read
    value is divided by a scaling factor of 1000.0 (the '.0' is important
    for Arduino to handle this as a float), resulting in a range of 0.000
    to 1.000.  The number of decimal places displayed is set in the
    lcdPrintDouble function.  It is set to '3' by default.

    The lcdBarGraph library allows the drawValue function to set the 
    maximum value displayed.  In this case, full-scale 
    (1.000 to 1.023) is set to 1000.
  */
   analogIn = analogRead(sensorPin);    // Read the analog input value
   //analogIn = 1020;
   memset(buffer, lcdNumCols + 1, '\0');

   val = (float) analogIn / scaleFactor;
   dtostrf(val, VALUEWIDTH, DEFAULTPRECISION, buffer);
   len = strlen(buffer);
   strcat(buffer, " ");
   strcat(buffer, scale);
  
   lcd.setCursor(0,0);                  // Set up the display...
   lcd.print(buffer);
  
   if (analogIn <= MAXANALOGVALUE) {         // If analog value within scale...
     lbg.drawValue(analogIn, MAXANALOGVALUE);// show analog value of input read
     delay(UPDATEDELAY);    
   } else {                                  // ...otherwise complain about it.
     lcd.setCursor(0,1);
     lcd.print("   Overrange!   ");  
     lcd.noDisplay();                        // flash the display on and off 
     delay(ERRORDELAY);
     lcd.display();
     delay(ERRORDELAY);       
   } 
}


