/* Barnacle Budd's Morse Code Decoder v. 1.4
 (c) 2013, Budd Churchward - WB7FHC
 
 This project makes use a custom built tone decoder module using
 the LM567C microchip. Details of this module will eventually be posted
 on line. This module allows you to tune to the frequency of a specific
 tone while ignoring noice and other tones of different frequencies
 
 The program will automatically adjust to the speed of code that
 is being sent. The first few characters may come out wrong while it
 homes in on the speed. If you are not seeing solid copy, press the
 restart button on your Arduino. You can try adjusting the tone decoder.
 Lowering the volume of the incoming CW can also help. If the tone decoder
 is not centered on the frequency of the incomming signal, you may have
 to fine tune the module as you lower the volume.
 
 The software tracks the speed of the sender's dahs to make
 its adjustments. The more dahs you send at the beginning
 the sooner it locks into solid copy.
 
 After a reset, the following text is very difficult to lock in on:
 'SHE IS HIS SISTER' because there are only two dahs in the whole
 phrase and they come near the end. However, if you reset and then
 send 'CALL ME WOODY' it will match your speed quite quickly.
 
 This project is built around the 20x4 LCD display. The sketch includes
 funtions for word wrap and scrolling. If a word extends beyond the 20
 column line, it will drop down to the next line. When the bottom line
 is filled, all lines will scroll up one row and new text will continue
 to appear at the bottom.
 
 This version makes use of the 4 digit parallel method of driving the
 display. A two line, I2C version will be forthcoming.
 
 Hook up your LCD panel to the Arduino using these pins:
 LCD pin  1 to GND
 LCD pin  2 to +5V
 LCD pin  4 to D7
 LCD pin  6 to D6
 LCD pin 11 to D5
 LCD pin 12 to D4
 LCD pin 13 to D3
 LCD pin 14 to D2
 LCD pin 15 to +5V
 LCD pin 16 to GND
 
 Data from pin 8 of the LM567C will be fed to D8 on the Arduino
 When this pin is HIGH there is no tone detected.
 When this pin is LOW a tone of the set frequency has been detected.
 
 */

#include <LiquidCrystal.h>
#include <stdlib.h>
#include "IntegrationCode.h"
#include "Decoder.h"

#define DEBUG
// initialize the library with the numbers of the interface pins
//LiquidCrystal lcd(7, 6, 5, 4, 3, 2);
extern LiquidCrystal lcd;  // For our LCD shield

/////////////////////////////////////////////////////////////////////////////////

void Decoder::DecoderStartup(int pin) {
  audioPin = AUDIOPIN;
  audio = 1;
  memcpy(mySet, "##TEMNAIOGKDWRUS##QZYCXBJP#L#FVH09#8###7#####/-61#######2###3#45", 128);
  justDid = true; // Makes sure we only print one space during long gaps
  characterDone = true; // A full character has been sent
  ditOrDah = true;  // We have either a full dit or a full dah

  // The following values will auto adjust to the sender's speed
  averageDah = 240;     // A dah should be 3 times as long as a dit
  dit = 80;             // We start by defining a dit as 80 milliseconds
  realDit = dit;
  myBounce = 2;        // Used as a short delay between key up and down
  myNum = 0;           // We will turn dits and dahs into a binary number stored here

  downTime = 0;        // How long the tone was on in milliseconds
  upTime = 0;          // How long the tone was off in milliseconds
  startDownTime = 0;  // Arduino's internal timer when tone first comes on
  startUpTime = 0;    // Arduino's internal timer when tone first goes off

  pinMode(LEDPIN, OUTPUT);    // We're going to blink Arduino's onboard LED
 // lcd.clear();           // Get rid of any garbage that might appear on startup
 // lcd.print(" K&P CW Decoder");
 // lcd.setCursor(0, 1);
  memset(lcdBuffer, '\0', LCDCOLUMNS + 1);
}

void Decoder::DecoderPoll() {
  int passCount = 0;
  
  upTime = 0;
  while(true) {
    audio = digitalRead(audioPin); // What is the tone decoder doing?
    if (!audio) {
      keyIsDown();       // LOW, or 0, means tone is being decoded
      passCount = 0;
    }
    if (audio) {
      keyIsUp();          // HIGH, or 1
      passCount++;
    }
Serial.print("in decoder, audio = ");
Serial.print(audio);
Serial.print(" passCount = ");
Serial.println(passCount);
    if (passCount > NOSIGNALMAX && audio == 1) {  // No more code being received...
      lcd.clear();
      break;
    }
  }
}

/*****
 * This function is called when the uC senses the start of a Morse character. There are a number of
 * global variables set here.
 * 
 * Parameter list:
 * void
 * 
 * Return value:
 * void
 * 
 *****/

void Decoder::keyIsDown() {
  // The decoder is detecting our tone
  // The LEDs on the decoder and Arduino will blink on in unison
  digitalWrite(LEDPIN,1);            // turn on Arduino's LED

  if (startUpTime > 0){
    // We only need to do once, when the key first goes down
    startUpTime = 0;    // clear the 'Key Up' timer
  }
  if (startDownTime == 0) {
    startDownTime = millis();  // get Arduino's current clock time
  }

  characterDone = false; // we're still building a character
  ditOrDah = false;      // the key is still down we're not done with the tone
  delay(myBounce);     // Take a short breath here

  if (myNum == 0) {      // myNum will equal zero at the beginning of a character
    myNum = 1;          // This is our start bit - it only does this once per letter
  }
#ifdef DEBUG1
  Serial.print("At bottom of keydown: myNum = ");
  Serial.println(myNum);
#endif

}

/*****
 * This function is called when the uC senses the end of a Morse character by the absence of an
 * audio tone. Again, there are a number of global variables set here.
 * 
 * Parameter list:
 * void
 * 
 * Return value:
 * void
 * 
 *****/

void Decoder::keyIsUp() {
  int farnsworthCutoff;

  // If we haven't already started our timer, do it now
  if (startUpTime == 0){
    startUpTime = millis();
  }

  // Find out how long we've gone with no tone
  upTime = millis() - startUpTime;


  // If it is twice as long as a dah print a space
#ifdef DEBUG1
  Serial.print("upTime = ");
  Serial.print(upTime);
  Serial.print(" dit = ");
  Serial.print(dit);
  Serial.print(" realDit = ");
  Serial.print(realDit);
  Serial.print(" aveDah = ");
  Serial.print(averageDah);
  Serial.print(" myNum = ");
  Serial.print(myNum);
  Serial.print(" lcdGuy = ");
  Serial.print(lcdGuy);
  Serial.print(" m[] = ");
  Serial.print(mySet[myNum]);
  Serial.print("  farn = ");
  Serial.println(farnsworthCutoff);
#endif
  if (upTime < FALSEREAD)              // Static?
    return;

  if (realDit < 90) {
    farnsworthCutoff = averageDah * 2;
  } 
  else {
    if (realDit > 90 && realDit < 105) {
      farnsworthCutoff = averageDah + realDit;
    } 
    else {
      farnsworthCutoff = averageDah * 3;
    }
  }
#ifdef DEBUG1
  Serial.println("");
  Serial.print("upTime = ");
  Serial.print(upTime);
  Serial.print("   dit = ");
  Serial.print(dit);
  Serial.print("   realDit = ");
  Serial.print(realDit);
  Serial.print("   avedah = ");
  Serial.print(averageDah);
  Serial.print("   farn = ");
  Serial.print(farnsworthCutoff);
  Serial.print("  lcdGuy = ");
  Serial.println(lcdGuy);
#endif

  //if (upTime > (averageDah * 2)) { 
  if (upTime > (realDit * 3)) {         // ***** PROBLEM AREA
    //if (upTime > farnsworthCutoff) {
    //if (upTime > dit) {
    printSpace();
  }

  // Only do this once after the key goes up
  if (startDownTime > 0){
    downTime = millis() - startDownTime;  // how long was the tone on?
    startDownTime = 0;                    // clear the 'Key Down' timer
  }

  if (!ditOrDah) {   // We don't know if it was a dit or a dah yet
    shiftBits();    // let's go find out! And do our Magic with the bits
  }

  // If we are still building a character ...
  if (!characterDone) {       // Are we done yet?
#ifdef DEBUG1
    Serial.print("upTime = ");
    Serial.print(upTime);
    Serial.print("   dit = ");
    Serial.print(dit);
    Serial.print("   realDit = ");
    Serial.print(realDit);
    Serial.print("   myNum = ");
    Serial.print(myNum);
    Serial.print("   lcdGuy = ");
    Serial.print(lcdGuy);
    Serial.print("   m[] = ");
    Serial.println(mySet[myNum]);
#endif

    //if (upTime > dit) { 
    if (upTime > realDit) {      // BINGO! we're done with this one  
      printCharacter();         // Go figure out what character it was and print it       
      characterDone = true;     // We got him, we're done here
      myNum = 0;                // This sets us up for getting the next start bit
    }
    downTime = 0;                // Reset our keyDown counter
  }

}


void Decoder::shiftBits() {
  // we know we've got a dit or a dah, let's find out which
  // then we will shift the bits in myNum and then add 1 or not add 1

  if (downTime < dit / 3) 
    return;                // ignore QRN - UnREM this line if your have a real noisy receiver
  // it will limit the top speed that you can copy.


  myNum = myNum << 1;    // shift bits left
  ditOrDah = true;       // we will know which one in two lines 

#ifdef DEBUG1
  Serial.print("down = ");
  Serial.print(downTime);
  Serial.print("   dit = ");
  Serial.print(dit);
  Serial.print("   realDit = ");
  Serial.print(realDit);
  Serial.print("   avedah = ");
  Serial.print(averageDah);
  Serial.print("  lcdGuy = ");
  Serial.println(lcdGuy);
#endif

  // If it is a dit we add 1. If it is a dah we do nothing!
  if (downTime < dit) {
    //if (downTime < realDit * 2) {    
    myNum++;           // add one because it is a dit
    realDit = (downTime + realDit) / 2;
    //averageDah = realDit * 3;
    //dit = realDit;
  } 
  else { 
    // The next three lines handle the automatic speed adjustment:
    averageDah = (downTime + averageDah) / 2;  // running average of dahs
    dit = averageDah / 3;                      // normal dit would be this
    realDit = dit;                             // Track this for timing
    dit = dit * 2;                             // double for threshold between dits and dahs
  }
}


void Decoder::printCharacter() {           
  justDid = false;         // OK to print a space again after this
#ifdef DEBUG1
  static int i = 0;
  if (i == 30) {
    Serial.println("");
    i = 0;
  } 
  else {
    // Serial.print(' ');
    // Serial.print(myNum);
    // Serial.print(' ');
    // Serial.print(lcdGuy);
    i++;
  }
#endif

  // Punctuation marks will make a BIG myNum
  if (myNum > 63) {  
    printPunctuation();  // The value we parsed is bigger than our character array
    // It is probably a punctuation mark so go figure it out.
    return;              // Go back to the main loop(), we're done here.
  }
  lcdGuy = mySet[myNum]; // Find the letter in the character set

  DoMyLCD();           // Go figure out where to put in on the display
}

void Decoder::printSpace() {
  if (justDid) {
    return;  // only one space, no matter how long the gap
  }

  justDid = true;       // so we don't do this twice  
  lcdGuy = ' ';            // this is going to go to the LCD 
  DoMyLCD();         // go figure out where to put it on the display

}

/*****
 * Punctuation marks are made up of more dits and dahs than letters and numbers. 
 * Rather than extend the character array out to reach these higher numbers we 
 * will simply check for them here. This funtion only gets called when myNum is 
 * greater than 63.
 * 
 * Parameter List:
 * void
 * 
 * Return value:
 * void
 *****/
void Decoder::printPunctuation() {
  switch (myNum) {
  case 71:
    lcdGuy = ':';
    break;
  case 76:
    lcdGuy = ',';
    break;
  case 84:
    lcdGuy = '!';
    break;
  case 94:
    lcdGuy = '-';
    break;
  case 97:
    lcdGuy = 39;    // Apostrophe
    break;
  case 101:
    lcdGuy = '@';
    break;
  case 106:
    lcdGuy = '.';
    break;
  case 115:
    lcdGuy = '?';
    break;
  case 246:
    lcdGuy = '$';
    break;
  case 122:
    lcdGuy = 's';
    DoMyLCD();
    lcdGuy = 'k';
    break;
  default:
    lcdGuy = '#';    // Should not get here
    break;
  }

  DoMyLCD();    // go figure out where to put it on the display
}

/*****
 * This function moves the current character read to the LCD display. It assumes that
 * lcdGuy has been set prior to calling this function.
 * 
 * Parameter list:
 * void
 * 
 * Return value:
 * void
 * 
 *****/
void Decoder::DoMyLCD() {
  static int passCounter = 0;
  int wpm;
  char numBuff[5];
  char outBuff[17];
#ifdef DEBUG1
  Serial.print(lcdGuy);
#endif

  if (passCounter % 100 == 0) {
#ifdef DEBUG1
    Serial.println("");
    Serial.print("***** realDit = ");
    Serial.print(realDit);
    Serial.print(" averageDah = ");
    Serial.print(averageDah);
    Serial.print("   dit = ");
    Serial.println(dit);
#endif
    wpm = 1200 / realDit + (realDit * .06);
    itoa(wpm, numBuff, 10);
    strcpy(outBuff, "Approx WPM = ");
    strcat(outBuff, numBuff);
    strcat(outBuff, " ");
    lcd.setCursor(0,0);
    lcd.print(outBuff);
    passCounter = 0;
  }
  passCounter++;
  memcpy(lcdBuffer, &lcdBuffer[1], LCDCOLUMNS - 1);
  lcdBuffer[LCDCOLUMNS - 1] = lcdGuy;
  lcd.setCursor(0, 1);
  lcd.print(lcdBuffer);
}



