/******
Flexible 4-Port Sequencer  Version 1.0

Designed by Dennis Kidder W6DQ 5 Jan 2014

This sketch when used with the purpose-built hardware described in the 
book, "Arduino Projects for Ham Radio," will provide a completely 
configurable sequencer.  Two sets of two arrays configure the ON sequence 
the OFF sequence and the delays between each step.  This allows the ON 
and OFF sequence order to differ along with the ON and OFF times for each
step through sequence.
******/

/******
The sequencer uses the following Digispark digital IO pins:

	P0 – Relay K1
	P1 – Relay K2
	P2 – Relay K3
	P4 – Relay K4

	P3 – PTT Input Indicator LED
	P5 – PTT Input

The following four lines of code allow the user to configure the ON and OFF
sequence and delay times for each step.  The default configuration turns
the relays on in the sequence "1,2,3,4" (represented logically by digital
IO pins 0, 1, 2, and 4) and to turn them off in the reverse order "4,3,2,1."

The delay between steps is expressed in milliseconds and the default is 50.
The order of the values in the array corresponds to the order in which the 
relays are turned ON or OFF.  Note that the first entry (default 40) takes
into consideration that the debounce delay of 10 mS must be included in the
total delay time, e.g. 50 mS (total) minus 10 mS (debounce delay) equals 40
milliseconds.  The remaining time delays remain unmodified.
******/

#define DEBOUNCEDELAY  10

int relayPin[] = {0,1,2,4};         // order to sequence relays on
int onDelay[] = {40,50,50,50};      // ON times in sequence

int relayPinOff[] = {4,2,1,0};      // order to sequence relays off
int offDelay[] = {40,50,50,50};     // OFF times in sequence

int inputInd = 3;                   // pin 3 drives the input indicator LED
int pttInput = 5;                   // pin 5 is the PTT input

int arrayElements = (sizeof(relayPin) / sizeof(relayPin[0])); //  elements

void setup()
{

/******
The following for loop is used to setup each of the pins in the relayPin[] array as an output.
******/

  for(int index = 0; index < arrayElements; index++)
  {
    pinMode(relayPin[index], OUTPUT);  //Set the relayPins as outputs
  } 

  pinMode(inputInd, OUTPUT);        // PTT input indicator
  pinMode(pttInput, INPUT);         // PTT input
    
  digitalWrite(pttInput, HIGH);     // enable the internal pullup resistor
 
}

void loop()
{
  
/******
The following sequence calls the debounce() function to determine if there 
Is a TRUE input on pttInput.

If pttInput is TRUE, the Input LED (inputInd) is lit, and the for loop uses
“index” to step through the onDelay[] and relayPin[] arrays, applying the
delay to the associated output pin.

When pttInput goes FALSE the Input LED is extinguished and the next for loop
Uses index to step through the offDelay[] and relayPin[] arrays, applying the
off delay for each relay in the order specified in the relayPin[] array.

******/
  if (debounce(pttInput) == true) {
    digitalWrite(inputInd, HIGH);    
    for(int index = 0; index < arrayElements; index++)
    {
      delay(onDelay[index]); 
      digitalWrite(relayPin[index], HIGH);    
    }
  } else {
    digitalWrite(inputInd, LOW);
    for (int index = 0; index < arrayElements; index++)
    {    
      delay(offDelay[index]);
      digitalWrite(relayPinOff[index], LOW);
    }
  }
}

/*****
  This function is a pretty standard debounce function so we can determine
  that the switch has really been pressed.
  
  Parameter list:
    int pin            the pin that registers the switch state
    
  Return value:
    boolean            true if the switch was pressed, false otherwise
    
*****/
boolean debounce(int pin)
{
  boolean currentState;
  boolean previousState;
  int i; 
  
  previousState = digitalRead(pin);
  for (i = 0; i < DEBOUNCEDELAY; i++) {
    delay(1);    // small delay
    currentState = digitalRead(pin);  // Read it now
    if (currentState != previousState)
    {
      i = 0;
      previousState = currentState;
    }
  }
  if (currentState == LOW)
    return true;
  else
    return false;
}