
/*  
 Jack Purdum and Dennis Kidder:
   Rev 1.10:  May 4, 2014
*/

//#include <pins.h>
#include <avr/pgmspace.h>
#include <EEPROM.h>
#include <LiquidCrystal.h>
#include <rotary.h>
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include "DDSLCDShield.h"
#include "W6DQ_DDS_VFO.h"

#define TITLE                 "ROTATOR CONTROLR"
#define VERSION               "1.10"
#define DATE                  "030514"

//#define INITIALIZEEEPROM 1      // Uncomment the first time you run the software
//#define DEBUG 1

#define HowBigIsArray(x)      (sizeof(x) / sizeof(x[0]))

#define ROTATORSENSORPIN      1 
#define PWMPIN                5
#define BRAKERELAYPIN         7 
#define CCWRELAYPIN           8
#define CWRELAYPIN            9

#define STARTDELAY         1000    // Splash screen display
#define READDELAY           500    // Enough time to get a beam heading reading
#define ABORTDELAY         2000    // Delay after abort
#define DELTAMOVE            10    // A minimum movement in rotating the beam
#define BRAKEDELAY          100    // Wait for brake release
#define DUTYCYCLE           240
#define YAESUCROSSOVER      816    // analog value when past 360 degrees
#define YAESUNORTH          408    // linear scale for North
//=================================== Rotator Type ======================================
#define TEST                  0
#define CDE                   1
#define HYGAIN                2
#define TELEX                 3
#define MFJ                   4
#define YAESU                 5

//=================================== Switch configuration ==============================
#define STARTSWITCH           1    // SW1 on board
#define MEMORYSWITCH1         2    // SW2
#define MEMORYSWITCH2         4    // SW3
#define MEMORYSWITCH3         8    // SW4
#define MEMORYSWITCH4        16    // SW5
#define MEMORYSTORE          32    // SW6, encoder switch

//=================================== LCD configuration =================================
#define LCDCOLS              16
#define LCDROWS               2
#define CURRENTHEADINGOFFSET  8    // Place current heading in column 8, row 0
#define DEGREESOFFSET        12    // Place degrees in column 12, row 1
#define GOTOOFFSET            8    // Place the ending heading

//=================================== Encoder configuration =============================
#define ENCODERASLEEP         0    // The EEPROM encoder is just tuning
#define ENCODERREVIEW         1    // Scrollinig through user-stored EEPROM frequencies
#define ENCODERSTORE          2    // Ready to store a new frequency in user EEPROM
#define ENCODERCHANGED        3

//=================================== Beam configuration ================================
#define BRAKERELEASEDELAY  5000    // Five second delay after brake release

#define BEAMSTABLE            0
#define BEAMINTRANSIT         1
#define BEAMABORT             2

#define MOVEINITIALIZATION    0    // Prepare to move beam: release brake, wait 5 seconds
#define MOVEINMOTION          2    // The beam is in motion
#define MOVEATREST            5    // Movement has stopped, beam is locked

#define MOVEBEAMUP            1
#define MOVEBEAMNOWHERE       0
#define MOVEBEAMDOWN         -1

//=================================== Heading configuration =============================
#define CURRENTHEADING        0    // Holds the current heading being used
#define MEMORYHEADING1        1    // Each of the following is tied to its switch
#define MEMORYHEADING2        2
#define MEMORYHEADING3        3
#define MEMORYHEADING4        4
#define MANUALMOVE            5

#define ROTATIONIDLE          6
#define ROTATIONCW            7
#define ROTATIONCCW           8
#define ABORTROTATOR          9

char statusField[][10] = {"CUR ", "MEM1", "MEM2", "MEM3", "MEM4", "MAN", "IDLE", "CW", "CCW", "ABRT"};

#ifdef INITIALIZEEEPROM
int headings[]      = {90, 130, 200, 280, 325};  // Some trial headings, run once
#else
int headings[5];                                 // Used the rest of the time
#endif

int typeRotator;
int statusFieldIndex;
int currentGoto;
int beamStatus;        // What's the beam doing?
int beamDirection;
int beamMoveStatus;    // Where we are in move sequence
int linearGoto;
int linearCurrent;
int memorySelected;
int initialHeading;

int mapCurrent, mapGoto;    // Used to find rotations in CalculateDirection()
int convertCurrent, convertGoto;
int lastDisplay;
  
 
volatile int encoderStatus;
volatile int currentHeading;
volatile int oldHeading;


Rotary myRotary = Rotary(2,3);             // Rotary encoder pins.  Must be interrupt pins.
DDSLCDShield lcd = DDSLCDShield();

//=================================== setup() function ==================================
void setup() {
  analogReference(EXTERNAL);
   
              //These two lines are run once to establish some test records in EEPROM.             
#ifdef INITIALIZEEEPROM
  for (int i = 0; i < HowBigIsArray(headings); i++)
    writeEEPROMRecord(headings[i], i);
#endif

#ifdef DEBUG
Serial.begin(115200);            // Turn on serial link
#endif
  
  pinMode(BRAKERELAYPIN,OUTPUT);
  pinMode(CCWRELAYPIN,OUTPUT);
  pinMode(CWRELAYPIN,OUTPUT);
  pinMode(PWMPIN,OUTPUT);

  typeRotator = YAESU;           // *** User sets appropriate type here ***
  //typeRotator = HYGAIN;           // *** User sets appropriate type here ***

  ReadBeamHeading();             // Get the initial beam heading
  MapLinearCurrentToCurrentHeading();  // Fills in linearCurrent

  currentGoto = currentHeading = initialHeading;
  
  ReadEEPROMArray();                  // Read the headings[] array from EEPROM
  
  lcd.begin(LCDCOLS, LCDROWS);        // Display LCD
  
  SplashScreen();
  InitializeDisplay();
  
  beamStatus = BEAMSTABLE;
  beamDirection = MOVEBEAMNOWHERE;
  beamMoveStatus = MOVEATREST;
  encoderStatus = ENCODERASLEEP;
  
  memorySelected = MEMORYHEADING1;
  statusFieldIndex = ROTATIONIDLE;
  DisplayCurrentHeading();                    // Show all three fields
  DisplayDegrees(ROTATIONIDLE);
  DisplayGoto();
  
  
  cli();                                      // Disable interrupts
  PCICR |= (1 << PCIE2);                      // Interrupt service code for encoder
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();                                      // Enable interrupts
}


//=================================== loop() function ===================================
void loop() {
  uint8_t buttons = lcd.readButtons();    // See if they pressed a button
  delay(BOUNCEDELAY);

  ReadBeamHeading();
  if (lastDisplay != currentHeading) {   // Done to reduce flicker
    DisplayCurrentHeading();            // Show the user we're making progress...
    lastDisplay = currentHeading;
  } 

  if (buttons > 0 && beamMoveStatus == MOVEINMOTION) {  // They want to abort beam movement
    AbortBeamMove();    // The beam is in transit, but they presses a button.
    buttons = 0;
  }
 
  if (beamStatus == BEAMINTRANSIT && beamMoveStatus == MOVEINMOTION) {   // The beam is being moved...
    ReadBeamHeading();                  // Sets linearCurrent by reading beam sensor  
    currentHeading += beamDirection;
    MoveBeam();
    if (currentHeading > 359)          // Handle wrap-around
      currentHeading = 0;
    if (currentHeading < 0)
      currentHeading = 359;
    if (currentHeading == currentGoto) {  // Are we there??
      StopRotator();                    // Stop rotator sequence
      beamStatus = BEAMSTABLE;          // We're done
      beamMoveStatus = MOVEATREST;
      encoderStatus = ENCODERASLEEP;    // Encoder not being moved
      DisplayDegrees(ROTATIONIDLE);     // Rotator idle
    }
  }

  if (buttons && beamStatus != BEAMABORT) {  // Did they pressed a button, but not in ABORT state?
    ButtonPush(buttons);                     // Yep...
  }

  if (encoderStatus == ENCODERCHANGED) {
    statusFieldIndex = MANUALMOVE;         // They are setting the heading manually
    DisplayDegrees(statusFieldIndex);
    DisplayCurrentHeading();               // Show the user we're making progress...
    DisplayGoto();
    encoderStatus = ENCODERASLEEP;
  }
      // Figure out display fields...
  if (oldHeading != currentHeading && encoderStatus != ENCODERASLEEP) {
    oldHeading = currentHeading; 
    DisplayCurrentHeading();
  }
  if (oldHeading != currentGoto && beamStatus != BEAMSTABLE) {
    oldHeading = currentGoto;
    DisplayGoto();
  }
}

//=================================== Internal functions ================================

/*****
  This function takes the value of currentGoto and maps it to linearGoto
  
  Argument list:
    void
    
  Return value:
    void
*****/
void SetLinearGoto()
{
  linearCurrent = analogRead(ROTATORSENSORPIN);      // Where's the beam at the start?
  MapLinearCurrentToCurrentHeading();								 // Map linear to current...
  currentHeading = initialHeading;									 // ...and copy
#ifdef DEBUG
Serial.print("First SetLinear currentHeading = ");
Serial.print(currentHeading);
Serial.print("   linearCurrent = ");
Serial.println(linearCurrent);
#endif
  if (typeRotator != YAESU) {
		if (currentGoto >= 180)
		linearGoto = map(currentGoto, 180, 360, 0, 512);      // From degrees to rotator coordinates
		else
		linearGoto = map(currentGoto, 0, 179, 512, 1024);      // From degrees to rotator coordinates
  } else {
    
#ifdef DEBUG
//currentHeading = 179;
//currentGoto = 181;
//linearCurrent = map(currentHeading, 0, 270, YAESUNORTH, 1023);
Serial.print("First SetLinear currentHeading = ");
Serial.print(currentHeading);
Serial.print("   linearCurrent = ");
Serial.println(linearCurrent);
#endif
   //YAESUCROSSOVER = 816    analog value when past 360 degrees
  //YAESUNORTH = 408        linear scale for North
      if (currentHeading == currentGoto)
        return;
      if (linearCurrent >= YAESUCROSSOVER && currentGoto <= 270) { // Beam is in overlap, but Goto in range
        linearGoto = map(currentGoto, 180, 270, YAESUCROSSOVER, 1023);
#ifdef DEBUG
Serial.print("First test linearGoto = ");
Serial.println(linearGoto);
#endif
      } else {
        if (linearCurrent >= YAESUCROSSOVER && currentGoto > 270) { // In overlap, and Goto > 270
          linearGoto = map(currentGoto, 0, 360, 0, YAESUNORTH);
#ifdef DEBUG
Serial.print("2nd test linearGoto = ");
Serial.println(linearGoto);
#endif
        } else 
        if (linearCurrent <= YAESUNORTH && currentGoto >= 180) {     // Beam between 180-360, Goto > 180
          linearGoto = map(currentGoto, 180, 360, 0, YAESUNORTH);
#ifdef DEBUG
Serial.print("3rd test linearGoto = ");
Serial.println(linearGoto);
#endif
        } else if (linearCurrent <= YAESUNORTH && currentGoto <= 180) {   // Beam < 360, Goto > 0 && < 180
          linearGoto = map(currentGoto, 0, 180, YAESUNORTH, YAESUCROSSOVER);
#ifdef DEBUG
Serial.print("4th test linearGoto = ");
Serial.println(linearGoto);
#endif
        } else if (linearCurrent > YAESUNORTH && currentGoto <= 180) {   // Beam > 0, Goto > 0 && < 180
          linearGoto = map(currentGoto, 0, 180, YAESUNORTH, YAESUCROSSOVER);
        } else if (linearCurrent >= YAESUNORTH && currentGoto >= 180) { 
          linearGoto = map(currentGoto, 180, 450, YAESUCROSSOVER, 1023);   
        }       
      }
  }
#ifdef DEBUG
Serial.print("In bottom SetLinearGoto: currentHeading = ");
Serial.print(currentHeading);
Serial.print("   currentGoto = ");
Serial.print(currentGoto);
Serial.print("   linearCurrent = ");
Serial.print(linearCurrent);
Serial.print("   linearGoto = ");
Serial.println(linearGoto);
#endif
  DisplayGoto(); 
}
/*****
  This function reads the current heading for the beam and assigns that reading into linearCurrent
  
  Argument list:
    void
    
  Return value:
    void
*****/
void MapLinearCurrentToCurrentHeading()
{
  if (typeRotator == YAESU) {
    initialHeading = map(linearCurrent, 0, 1023, -180, 270);
    if ( initialHeading < 360 ){
      initialHeading = map(initialHeading, -180, -1, 180, 359);
    }
    if (initialHeading > 359) {
      initialHeading = initialHeading - 360;
    }
  } else {
    initialHeading = map(linearCurrent, 0, 1023, -180, 180);
    if (initialHeading <= 0 ){
      initialHeading = map(initialHeading, -180, 0, 180, 360);
    }
  }
}
/*****
  This function reads the current heading for the beam and assigns that reading into linearCurrent
  
  Argument list:
    void
    
  Return value:
    void
*****/
void ReadBeamHeading()
{
  linearCurrent = analogRead(ROTATORSENSORPIN);    // Find where beam is pointing
  MapLinearCurrentToCurrentHeading();              // Map for display update
  currentHeading = initialHeading;
}
/*****
  This function is used to map the rotator sensor readings to the current beam heading to
  the new beam heading.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void CalculateDirection()
{ 
  if (linearCurrent > linearGoto)
    beamDirection = MOVEBEAMDOWN;
  else  
    beamDirection = MOVEBEAMUP;
    
  beamStatus = BEAMINTRANSIT;
  beamMoveStatus = MOVEINITIALIZATION;

}


/*****
  This function is used to prepare the rotator for moving the beam.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void InitializeMoveSequence()
{
  if (typeRotator != YAESU) {             // Release brake for all but Yaesu
    digitalWrite(BRAKERELAYPIN, HIGH);    // Energize brake relay to release brake
    delay(BRAKEDELAY);
  }
  if (beamDirection < 0) {            // Move CCW by energizing relay
    digitalWrite(CCWRELAYPIN, HIGH);  // Start moving CCW
    digitalWrite(CWRELAYPIN, LOW);    // Just playing it safe
  } else {
    digitalWrite(CWRELAYPIN, HIGH);   // Start moving CW
    digitalWrite(CCWRELAYPIN, LOW);   // Still playing it safe
  }  
  beamMoveStatus = MOVEINMOTION;
}

/*****
  This function is used with the Yaesu rotator, since it wants to slow as it approaches the
  correct heading.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void MoveBeam()
{ 
  int headingDifference;
  if (typeRotator == YAESU) {    
      headingDifference = abs(currentHeading - currentGoto);  // How much left to turn?
      
      switch (headingDifference) {
        case 14:
        case 13:                              // Coast to a stop...
        case 12:
          analogWrite(PWMPIN, 184);
          break;
        case 11:
        case 10:                              // Coast to a stop...
        case 9:
          analogWrite(PWMPIN, 128);
          break;
        case 8:
        case 7:
        case 6:
          analogWrite(PWMPIN, 72);
          break;
        case 5:
        case 4:
        case 3:
          analogWrite(PWMPIN, 16);
          break;
        case 2:
        case 1:
        case 0:                              // Turn off relays
          if (beamDirection < 0) {            // Move CCW 
            digitalWrite(CCWRELAYPIN, LOW);
          } else {
            digitalWrite(CWRELAYPIN, LOW);   //Move CW
          }
          currentHeading = currentGoto;
          break;
        default:            // All other values, move fast
          analogWrite(PWMPIN, DUTYCYCLE);
          break;    
      }
  } 
  beamMoveStatus = MOVEINMOTION;
}

/*****
  This function is used to stop the rotator according to the needs of the specific rotator being used.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void StopRotator()
{
  digitalWrite(CWRELAYPIN, LOW);              // Won't hurt to turn them both off
  digitalWrite(CCWRELAYPIN, LOW);
  if (typeRotator != YAESU) {
    delay(BRAKEDELAY);                        // Wait for it to settle down...
    digitalWrite(BRAKERELAYPIN, HIGH);        // Lock the beam in place
  }
  oldHeading = currentHeading = currentGoto;  // All the same now
  beamStatus = BEAMSTABLE;
  beamMoveStatus = MOVEATREST;
  encoderStatus = ENCODERASLEEP;
  DisplayCurrentHeading();               // We're there

}
/*****
  This function is called if the beam is being moved but the user presses any other switch.
  This is interpreted as an Abort of the beam movement.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void Abort()
{
  DisplayDegrees(ABORTROTATOR);
  oldHeading = currentGoto = currentHeading;
  if (beamStatus == BEAMINTRANSIT) {
    StopRotator();
  }
  beamStatus = BEAMABORT;
  beamMoveStatus == MOVEATREST;
  DisplayCurrentHeading();
  DisplayGoto();
}



/*****
  This function updates the affected fields when a memory stored heading is invoked
  
  Argument list:
    void
  
  Return value:
    void
*****/
void UpdateMemoryHeadingDisplays()
{
  DisplayDegrees(statusFieldIndex);
  currentGoto = headings[statusFieldIndex];
  DisplayGoto(); 
  DisplayCurrentHeading();   
}
/*****
  This function is used to display the current heading
  
  Argument list:
    void
  
  Return value:
    void
*****/
void DisplayCurrentHeading()
{
  if (currentHeading < 0)
    currentHeading = 360 + currentHeading;
  lcd.setCursor(CURRENTHEADINGOFFSET, 0);
  lcd.print("   ");
  if (currentHeading > 99)                      // Right-justify in field
    lcd.setCursor(CURRENTHEADINGOFFSET, 0);
  else if (currentHeading > 9)
      lcd.setCursor(CURRENTHEADINGOFFSET + 1, 0);
    else 
      lcd.setCursor(CURRENTHEADINGOFFSET + 2, 0);    
  lcd.print(currentHeading);
}

/*****
  This function is used to display the degrees 
  
  Argument list:
    int which    the index into the statusField array
  
  Return value:
    void
*****/
void DisplayDegrees(int which)
{
  lcd.setCursor(DEGREESOFFSET, 1);
  lcd.print("    ");
  lcd.setCursor(DEGREESOFFSET, 1);
  lcd.print(statusField[which]);
}

/*****
  This function is used to display the Go To heading 
  
  Argument list:
    void
  
  Return value:
    void
*****/
void DisplayGoto()
{
  if (currentGoto < 0)
    currentGoto = 360 + currentGoto;
    
  lcd.setCursor(GOTOOFFSET, 1);
  lcd.print("   ");
  if (currentGoto > 99)                      // Right-justify in field
    lcd.setCursor(GOTOOFFSET, 1);
  else if (currentGoto > 9)
      lcd.setCursor(GOTOOFFSET + 1, 1);
    else 
      lcd.setCursor(GOTOOFFSET + 2, 1);    
  lcd.print(currentGoto);
}


/*****
  This function is used to set the LCD display fields when the system is first powered up.
  
  Argument list:
    void
  
  Return value:
    void
*****/
void InitializeDisplay()
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("CURRENT:    DEG");
  lcd.setCursor(0, 1);
  lcd.print("  GO TO:");
}


/*****
  This function returns the stored heading.
  
  Argument list:
    int which      which of the 5 available headings to return
  
  Return value:
    int            the heading or -1 on error
*****/
int ReadHeading(int which)
{

  if (which >= HowBigIsArray(headings))
    return -1;
  return headings[which];
}

/*****
  This function writes the current contents of the headings[] array to EEPROM.
  
  Argument list:
    void
  
  Return value:
    void
    
*****/
void ReadEEPROMArray()
{
  int i;
  
  for (i = 0; i < HowBigIsArray(headings); i++) {
    headings[i] = readEEPROMRecord(i);
  }
}
/*****
  This method is used to read a record from EEPROM. Each record is 4 bytes (sizeof(unsigned long)) and
  is used to calculate where to read from EEPROM.
  
  Argument list:
    int record                the record to be read. While tuning, it is record 0
  
  Return value:
    unsigned long            the value of the record,
    
  CAUTION:  Record 0 is the current frequency while tuning, etc. Record 1 is the number of stored
            frequencies the user has set. Therefore, the stored frequencies list starts with record 23.
*****/
unsigned int readEEPROMRecord(int record)
{
  int offset;
  union {
    byte array[2];
    int val;
  } myUnion;
  
  offset = record * sizeof(int);

  myUnion.array[0] = EEPROM.read(offset);
  myUnion.array[1] = EEPROM.read(offset + 1);

  return myUnion.val;
}


/*****
  This method is write the heading data to EEPROM
  
  Argument list:
    int heading        the heading to save
    int record         the record where to store the heading
  
  Return value:
    void
*****/
void writeEEPROMRecord(int heading, int record)
{
  int offset;
  union {
    byte array[2];
    int val;
  } myUnion;
 
  if (heading < 0 || heading > 360) {   // Can't have negative headings or greater than 360 degrees
    Error();
    return;                          
  }
  myUnion.val = heading;
  offset = record * sizeof(int);

  EEPROM.write(offset, myUnion.array[0]);
  EEPROM.write(offset + 1, myUnion.array[1]);
}


/*****
  This function is a general purpose error indicator and can be used for any error condition,
  being it improper input or electronic malfunction. The word ERR is display in the DEG position
  on the LCD display.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void Error() 
{
  lcd.setCursor(DEGREESOFFSET, 1);
  lcd.print("ERR");
  delay(2000);
}

/*****
  This function is called if the user has started a beam move but decides to cancel it.
  
  Argument list:
    void
    
  Return value:
    void
*****/
void AbortBeamMove()
{
    Abort();
    delay(ABORTDELAY);                      // Let them see ABRT message
    beamStatus = BEAMSTABLE;          // We're done
    beamMoveStatus = MOVEATREST;
    statusFieldIndex = ROTATIONIDLE;
    encoderStatus = ENCODERASLEEP;
    oldHeading = currentHeading = currentGoto;  // All the same now
    DisplayCurrentHeading();               // We're there
    DisplayDegrees(ROTATIONIDLE);      

}

/*****
  This is the Interrupt Service Routine that is executed anytime the rotary encoded is turned.
  
  Argument list:
    PCINT2_vect    the vector for the interrupt routine
    
  Return value:
    void
*****/
ISR(PCINT2_vect) {
  unsigned char result = myRotary.process();

  if (beamStatus == BEAMINTRANSIT)          // No change during transit; only an abort
    return;
  switch (result) {
    case 0:                                 // Nothing done...
      return;
      
    case DIR_CCW:                            // Turning Clockwise, higher frequencies
      currentGoto++;
      if (currentGoto >= 360)
        currentGoto = 0;
      break; 

    case DIR_CW:                          // Turning Counter-Clockwise, lower frequencies    
      currentGoto--;  
      if (currentGoto < 0)
        currentGoto = 359;
       break;    
    default:
       // Should never be here
      break;
  }
  encoderStatus = ENCODERCHANGED;          // They are rotating the encoder shaft
}

/*****
  This method is used to process the various button presses by the user.
  
  Argument list:
    uint8_t button    Will be either BANDUPMODE or BANDDOWNMODE
  
  Return value:
    void
*****/
void ButtonPush(uint8_t button)
{
  switch (button) {
    case STARTSWITCH:            // Start rotator sequence 
      ReadBeamHeading();         // Sets linearCurrent
      SetLinearGoto();           // Sets the linearGoto value by mapping from degrees to linear
      if (currentHeading == currentGoto) {
        DisplayDegrees(ROTATIONIDLE);
        break;
      }  
      CalculateDirection();      // Which way to turn beam
      InitializeMoveSequence();
      if (beamDirection < 0)
        DisplayDegrees(ROTATIONCCW);
      else
        DisplayDegrees(ROTATIONCW);        
      break;
      
    case MEMORYSWITCH1:                 // Store current SWITCHheading in Memory location 1
      memorySelected = statusFieldIndex = MEMORYHEADING1;
      UpdateMemoryHeadingDisplays();   
      break;
      
    case MEMORYSWITCH2:                 // Store current heading in Memory location 2
      memorySelected = statusFieldIndex = MEMORYHEADING2;  
      UpdateMemoryHeadingDisplays();   
      break;
      
    case MEMORYSWITCH3:                 // Store current heading in Memory location 3
      memorySelected = statusFieldIndex = MEMORYHEADING3;
      UpdateMemoryHeadingDisplays();   
      break;
      
    case MEMORYSWITCH4:                 // Store current heading in Memory location 4
      memorySelected = statusFieldIndex = MEMORYHEADING4;
      UpdateMemoryHeadingDisplays();   
      break;

    case MEMORYSTORE:                     // Encoder switch 
      if (memorySelected < 1 || memorySelected > 4) {
        Error();     
        break; 
      }
      headings[memorySelected] = currentGoto;          // Assign new heading
      writeEEPROMRecord(currentGoto, memorySelected);  // Write it to EEPROM  
      beamStatus = BEAMSTABLE;                           // Beam is not necessarily moving
      break;
      
    default:    // ERROR
      Error();     
      break;
  }
}
/*****************
*
* This function prints the version number and date of the software
*
* Argument list
*   void
*
* Return value
*   void
*
******************/
void SplashScreen()
{  
  lcd.home();
  lcd.print(TITLE);
  lcd.setCursor(0,1);
  lcd.print("VER: ");
  lcd.setCursor(5,1);
  lcd.print(VERSION);
  lcd.setCursor(10,1);
  lcd.print(DATE);
  delay(STARTDELAY);
}
  

