/******
*
*   Directional Wattmeter / SWR Indicator by D. Kidder W6DQ
*   Version 1.7 26 Aug 2014
*   
*   Works with the hardware described in "Arduino Projects for Ham Radio"
*   by Jack Purdum, W9NMT, and Dennis Kidder, W6DQ.
*
*   The hardware uses a Stockton Coupler with AD8307 Log-Lin Amplifier as
*   a sensor.  As designed, the Wattmeter is designed for 25 Watts max but
*   will produce an alarm if the power exceeds 20 Watts.  With a backlit
*   RGB LCD, when in alarm, the backlight is red.  An audible alert is also
*   used.
*
*   The SWR indicator can be switched between two modes - one with a digital
*   indication, and the second with a bar graph display.  THe SWR indicator
*   also uses the LCD backlight to display status of the SWR.  With a low SWR
*   the display is green.  As the SWR begins to rise, the discplay is yellow.
*   A high SWR condition is shown as red with an audible alert.
*
******/

#include <LiquidCrystal.h>
#include <LcdBarGraph.h>
#include <math.h>
#include "TimerOne.h"

//===================================== LCD configuration ====================================================
#define LCDCOLS              16        // 16 columns
#define LCDROWS              2         // 2 rows

#define BARGRAPHROW          1         // Draw graph on second row
#define BARGRAPHCOL          0         // Start graph in column 0

#define DISPLAYFLASHRATE    300000    // Update rate in microseconds

#define POWERWIDTH           5         // Numeric field width, include '-', '.'
#define SWRWIDTH             4         // Numeric field width, include '-', '.'
#define DEFAULTPRECISION     1         // Number of decimal places to display
#define PRECISIONTWO         2         // Number of decimal places to display
#define PRECISIONZERO        0         // Number of decimal places to display

#define STARTUPDELAY         3000      // Display splash screen
#define DEBOUNCEDELAY        50

//===================================== Define inputs and outputs ============================================

#define FORWARDPOWERINPUT    1         // Forward power sensor: analog input 1
#define REFLECTEDPOWERINPUT  2         // Reflected power sensor: analog input 2
#define SWITCH               7         // Display mode switch: digital pin 7
#define BLUELIGHT            8         // Display backlight BLUE digital pin 8
#define GREENLIGHT           9         // Display backlight GREEN digital pin 9
#define REDLIGHT             10        // Display backlight RED digital pin 10
 
/******
*
*   The following eight values are modifiable by the user.  They are used to derive
*   constants used to calculate the input power. The values consist of two input
*   power levels and the corresponding analog input value for both forward and
*   reverse measurements.  In the case of *POWERONE, we want the analog value to
*   be the max before alarm.  We have arbitrarily used "1000" so that anything
*   great than 1000 (in other words 1000 to 1023) will cause an alarm.
*
******/

#define FORWARDPOWERONE      20.0       //  Forward power at max reading
#define FORWARDANALOGONE     1000       //  Analog value at max forward
#define FORWARDPOWERTWO      .020       //  Forward power at second reading
#define FORWARDANALOGTWO     706        //  Analog value at second forward
#define REFLECTEDPOWERONE    20.0       //  Reflected power at max reading
#define REFLECTEDANALOGONE   1000       //  Analog value at max refelected
#define REFLECTEDPOWERTWO    .020       //  Refelected power at second reading
#define REFLECTEDANALOGTWO   701        //  Analog vale at second reflected

//===================================== Alarm Levels =========================================================
#define POWERLEVELALARM      1000       //  Analog value for power that generates an alarm (we fudge it a little bit)
#define POWERLEVELALARMBAD   1010       //  Start flashing the display and make noise!
#define SWRYELLOW            5.0        //  Threshold for caution SWR
#define HIGHSWR              10.0       //  Threshold for SWR that is OFF THE CHARTS
#define SWRALARMTHRESHOLD    10.0       //  Threshold to disable the alarm functions (dBm)

boolean alarmLock = false;              //  Used to suppress display during hi power alarm states
boolean lowPowerAlarmLock = false;      //  Used to suppres alarms when power is very low
boolean hiSwrLock = false;              //  Used to suppress display during high SWR alarm states

#define ALARM1               1          //  Power exceeds POWERLEVELALARM
#define ALARM2               2          //  Power exceeds POWERLEVELALARMBAD
#define ALARM3               3          //  Reflected power > Forward power
#define ALARM4               4          //  SWR Rising
#define ALARM5               5          //  High SWR -- off the charts!!


//=========================== Define the different backlight modes for the LCD ===============================

#define GOWHITE   {digitalWrite(REDLIGHT, LOW); digitalWrite(GREENLIGHT, LOW);digitalWrite(BLUELIGHT, LOW);}
#define GOBLACK   {digitalWrite(REDLIGHT, HIGH); digitalWrite(GREENLIGHT, HIGH);digitalWrite(BLUELIGHT, HIGH);}
#define GORED     {digitalWrite(REDLIGHT, LOW); digitalWrite(GREENLIGHT, HIGH);digitalWrite(BLUELIGHT, HIGH);}
#define GOYELLOW  {digitalWrite(REDLIGHT, LOW);digitalWrite(GREENLIGHT, LOW); digitalWrite(BLUELIGHT, HIGH);}
#define GOGREEN   {digitalWrite(REDLIGHT, HIGH);digitalWrite(GREENLIGHT, LOW); digitalWrite(BLUELIGHT, HIGH);}

//===================================== Analog Input Buffer setup ============================================

#define BUFFERSIZE  10                         // Depth of the circular buffer

int index;                                     // Index into the buffer array

int forwardBufferTotal;                        // The running sum of the forward power buffer
int reflectedBufferTotal;                      // The running sum of the reflected power buffer

int forwardPowerCircularBuffer[BUFFERSIZE];    // Forward power buffer
int reflectedPowerCircularBuffer[BUFFERSIZE];  // Reflected power buffer

//===================================== Power expressed in different units ===================================

#define Watts                 1L               // Used to scale power to Watts
#define mW                    1000L            // Used to scale power to milliatts
#define uW                    1000000L         // Used to scale power to microwatts

int forwardPowerLog = 0;                       // Log of forward power
int reflectedPowerLog = 0;                     // Log of reflected power

float forwardPower = 0.0;                      // Power (in Watts) with different scaling
float reflectedPower = 0.0;
float forwardPowermW = 0.0;                    // milliwatts
float reflectedPowermW  = 0.0;
float forwardPoweruW = 0.0;                    // microwatts
float reflectedPoweruW = 0.0;

float forwardPowerDbm;                        // dB milliwatts (dBm)
float reflectedPowerDbm;

//========================= Constants used to calculate power from log amp output ============================
float forwardPowerSlope;
float forwardPowerIntercept;
float reflectedPowerSlope;
float reflectedPowerIntercept;

float swr;
int   swrgraph;

int buttonState;
int buttoncount = 0;

volatile int alarmType;
volatile boolean state;
volatile boolean OneShotOnly;

// ================================ Set the parameters for the LCD and the bar graph =========================

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);            // For LCD display from Chapter 3
LcdBarGraph   lbg(&lcd, LCDCOLS, BARGRAPHCOL, BARGRAPHROW);

// ================================ Setup buffers used for creating displays =================================

char buffer[ LCDCOLS + 1 ];
char lineOneBufferout[ LCDCOLS + 1 ];

// ============================================================================================================
// ================================ setup() Portion of the program ============================================
// ============================================================================================================

void setup()
{
// ================================ Initialize IO Pins =========================================================

Serial.begin(115200);

  pinMode(SWITCH,INPUT);              // Assign the display selector switch and turn on the pullup
  digitalWrite(SWITCH,HIGH);

  pinMode(REDLIGHT,OUTPUT);            // set the mode of the pins to drive the LCD backlight
  pinMode(GREENLIGHT,OUTPUT);
  pinMode(BLUELIGHT,OUTPUT);

  analogReference(EXTERNAL);                     // use ext reference from LM324 voltage follower (~3.7 VDC)

// ================================ Initialize the display plus add splash screen =============================

  GOWHITE;                            // Turn on the backlight!
  lcd.begin(LCDROWS, LCDCOLS);        // initialize the LCD
  lcd.clear();
  lcd.home();
  lcd.print("Watt/SWR Meter");        //sappy stuff, name, version and date
  lcd.setCursor(0,1);
  lcd.print("ver 1.7 26AUG14");
  delay(STARTUPDELAY);                // Delay probably okay since we're not doing anything else
  lcd.clear();

// ================================ Initialize Timer#1 to flash the display in alarm states ==================

  Timer1.initialize(DISPLAYFLASHRATE); 

/* ================================ Initialize slope and intercept values ====================================
*
*   Derive the slope and intercepts for the forward and reflected power curves.  Using the slope and 
*   intercept, we can derive the power with any given analog input from the sensors.
*  
*     Slope = (analog1 - analog2) / (power1 - power2) 
*  
*     Intercept = power1 - (analog1 / slope )
*  
*   power is expressed in dBm, analog is the analog input value
*
*****/

  forwardPowerSlope = slope(FORWARDANALOGONE, FORWARDPOWERONE, FORWARDANALOGTWO, FORWARDPOWERTWO);
  reflectedPowerSlope = slope(REFLECTEDANALOGONE, REFLECTEDPOWERONE, REFLECTEDANALOGTWO, REFLECTEDPOWERTWO);
  
  forwardPowerIntercept = intercept( FORWARDPOWERONE, FORWARDANALOGONE, forwardPowerSlope );
  reflectedPowerIntercept = intercept( REFLECTEDPOWERONE, REFLECTEDANALOGONE, reflectedPowerSlope );

// ================================ Initialize the input circular buffers to zero ============================

  memset(forwardPowerCircularBuffer, 0, BUFFERSIZE);
  memset(reflectedPowerCircularBuffer, 0, BUFFERSIZE);

/* 
  for(int index = 0; index < BUFFERSIZE; index++) {  // initialize the buffers with zeros
    forwardPowerCircularBuffer[index] = 0;
    reflectedPowerCircularBuffer[index] = 0;

  }
*/}
// ===========================================================================================================
// ================================ loop() Portion of the program ============================================
// ===========================================================================================================

void loop()
{
/* ================================ Circular Input Buffer and Averaging Code =================================  
*
*   In order to stabilze the radings from the sensors, we use a circular buffer.  The buffer is
*   an array that is of size BUFFERSIZE.  We read from the analog inputs into the buffer and then
*   after each value is added, sum the values in the buffer and divide by BUFFERZSIZE, giving an
*   average value.
*
*   In adddition, we monitor each reading to ensure that the maximum power levels are not exceeded.  Should
*   the instantaneous input power exceed preset limits, an alarm is raised and the display goes yellow,
*   indicating that the power is rising to a dangerous level, or flashing red, indicating that the power
*   is sufficiently high as to cause permanent damage to the sensor. Returning to normal levels removes
*   the alarm.
*
*****/
    
      forwardBufferTotal = forwardBufferTotal - forwardPowerCircularBuffer[index];
      forwardPowerCircularBuffer[index] = analogRead(FORWARDPOWERINPUT);
      forwardBufferTotal = forwardBufferTotal + forwardPowerCircularBuffer[index];

      reflectedBufferTotal = reflectedBufferTotal - reflectedPowerCircularBuffer[index];
      reflectedPowerCircularBuffer[index] = analogRead(REFLECTEDPOWERINPUT);
      reflectedBufferTotal = reflectedBufferTotal + reflectedPowerCircularBuffer[index];
      
// This section of code is used to detect alarm conditions ... excessive power ===============================
      
      if ( forwardPowerCircularBuffer[index] > POWERLEVELALARMBAD || reflectedPowerCircularBuffer[index] > POWERLEVELALARMBAD ){
        Timer1.attachInterrupt(FlashDisplay);
        alarmType = ALARM2;
        DisplayAlarm(alarmType);
        alarmLock = true;
      }
      else if ( forwardPowerCircularBuffer[index] > POWERLEVELALARM || reflectedPowerCircularBuffer[index] > POWERLEVELALARM ){
        Timer1.attachInterrupt(FlashDisplay);
        alarmType = ALARM1;
        DisplayAlarm(alarmType);
        alarmLock = true;
      }
      else {
        alarmLock = false;
        Timer1.detachInterrupt();
      }
      
// Alarm cleared, back to normal =============================================================================

      index++;                                    
      
      if ( index >= BUFFERSIZE ) index = 0;
      
      forwardPowerLog = forwardBufferTotal / BUFFERSIZE;
      reflectedPowerLog = reflectedBufferTotal / BUFFERSIZE;
 
// ================================ Series of conversions from the log output ================================

/********
*
*   Convert the log output of the sensor AD8307 to the actual power in dBm.  
*
******/

    forwardPowerDbm = ( forwardPowerLog / forwardPowerSlope ) + forwardPowerIntercept;
    reflectedPowerDbm = ( reflectedPowerLog / reflectedPowerSlope ) + reflectedPowerIntercept;

/*****
*
*   Convert the output power (dBm) to Watts
*
*****/

    forwardPower = dBmToWatts(forwardPowerDbm, Watts);
    reflectedPower = dBmToWatts(reflectedPowerDbm, Watts);

/*****
*
*   Convert the output power (dBm) to milliwatts
*
*****/

    forwardPowermW= dBmToWatts(forwardPowerDbm, mW);
    reflectedPowermW  = dBmToWatts(reflectedPowerDbm, mW);
    
/*****
*
*   Convert the output power (dBm) to microwatts
*
*****/

    forwardPoweruW = dBmToWatts(forwardPowerDbm, uW);
    reflectedPoweruW = dBmToWatts(reflectedPowerDbm, uW);    

/*****
*
*  This section creates a "lockout" on certain alarms.  When there is no power applied or
*  very low power levels, it is easy to put the code into an alarm state.  The lockout is used to 
*  suppres alarms until a minimum power level is reached.
*
*****/ 

    if ( forwardPowerDbm > SWRALARMTHRESHOLD || reflectedPowerDbm > SWRALARMTHRESHOLD ) {
      lowPowerAlarmLock = false;
    }
    else {
      lowPowerAlarmLock = true;
    }

/*****
*
*   Calculate the SWR from the forward and reflected power.  The relationship between
*   SWR and polwer is expressed as:
*
*      SWR = ( 1 + SQRT( Pref / Pfor)) / SQRT( 1 - ( Pref / Pfor))
*
*****/

    swr = ( 1 + sqrt( reflectedPowermW  / forwardPowermW)) / ( 1 - sqrt( reflectedPowermW  / forwardPowermW));
    swr = constrain(swr, 1.0, 20.0 );
    


// ================================ Pushbutton is used to select from four display modes =====================

    buttonState = digitalRead(SWITCH);
  
    if (buttonState == LOW) {
      deBounce(SWITCH);    
      buttoncount++;
      if ( OneShotOnly == false ) OneShotOnly = true;
      lcd.clear();
      GOWHITE;
    }

/******
*
*  This portion of the code is used to detect alarm states NOT detected previously.
*  This section deals with high SWR and/or reversed input and output.
*     1. High SWR. Result: flashing red dsiplay and error message
*     2. Reflected power exceeds forward power. Result: Flashing red dsiplay and error message
*
*  When the alarm condition is cleared, we continue to the normal display modes.  There
*  are four display modes:
*
*    1. Forward and Reflected power displayed in Watts, milliwats and microwatts.
*    2. Forward and Reflected power displayed in dBm.
*    3. SWR displayed on top line, bar graph on lower line. 3:1 is approx mid-scale.
*    4. Abbreviated Forward and Reflected power with SWR display.
*
*  Alarms are disabled at very low power levels.  Variations in the output of the log
*  amps can cause an unwanted alarm with no RF input power at all.  The value of ALARMTESTTHRESHOLD
*  sets the minimum amount of power to be applied before the alarm tests are enabled.
******/

    if (alarmLock != true ){

      if ( swr > HIGHSWR && lowPowerAlarmLock == false ){
        Timer1.attachInterrupt(FlashDisplay);
        alarmType = ALARM5;
        DisplayAlarm(alarmType);
        hiSwrLock = true;
      }
      else if ( swr > SWRYELLOW && lowPowerAlarmLock == false ){

        dtostrf(swr, SWRWIDTH, DEFAULTPRECISION, buffer);
        sprintf(lineOneBufferout,"  SWR: %s:1   ",buffer);
        Timer1.attachInterrupt(FlashDisplay);
        alarmType = ALARM4;
        DisplayAlarm(alarmType);
        hiSwrLock = true;
      }
      else if ( reflectedPowerDbm > forwardPowerDbm && lowPowerAlarmLock == false ){
        Timer1.attachInterrupt(FlashDisplay);
        alarmType = ALARM3;
        DisplayAlarm(alarmType);
        hiSwrLock = true;
    }
      else {
        hiSwrLock = false;
        Timer1.detachInterrupt();
    }

    if (hiSwrLock != true){  

      GOWHITE;
      
      switch (buttoncount){

        case 0:                           // Display No. 1: Forward and Reflected power in Watts
          
         if ( forwardPowerDbm > 40.0 ){
            dtostrf(forwardPower, POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"Forward:%s W ",buffer);      
          }
          else if ( forwardPowerDbm > 30.0 ){
            dtostrf(forwardPower, POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"Forward:%s W ",buffer);      
          }
          else if ( forwardPowerDbm > 20.0 ){
            dtostrf(forwardPowermW, POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"Forward:%s mW",buffer);
          }    
          else if ( forwardPowerDbm > 10.0 ){
            dtostrf(forwardPowermW, POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"Forward:%s mW",buffer);
          }
          else if ( forwardPowerDbm > 0.0 ){
            dtostrf(forwardPowermW, POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"Forward:%s mW",buffer);
          }
          else {
            dtostrf(forwardPoweruW, POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"Forward:%s uW",buffer);
          }

          lcd.home();
          lcd.print(lineOneBufferout);
  
          if ( reflectedPowerDbm > 40.0 ){      
            dtostrf(reflectedPower, POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"Reflctd:%s W ",buffer);      
          }
          else if ( reflectedPowerDbm > 30.0 ){      
            dtostrf(reflectedPower, POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"Reflctd:%s W ",buffer);      
          }
          else if ( reflectedPowerDbm > 20.0 ){    
            dtostrf(reflectedPowermW , POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"Reflctd:%s mW",buffer);
          }    
          else if ( reflectedPowerDbm > 10.0 ){    
            dtostrf(reflectedPowermW , POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"Reflctd:%s mW",buffer);
          }
          else if ( reflectedPowerDbm > 0.0 ){    
            dtostrf(reflectedPowermW , POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"Reflctd:%s mW",buffer);
          }
          else {    
            dtostrf(reflectedPoweruW, POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"Reflctd:%s uW",buffer);
          }

          lcd.setCursor(0,1);
          lcd.print(lineOneBufferout);
      
          break;

        case 1:                           // Display No. 2: Forward and Reflected power in dBm

          if ( forwardPowerDbm < -10.0 ){      
            dtostrf(forwardPowerDbm, POWERWIDTH - 1, PRECISIONZERO, buffer);
          }
          
          else{

            dtostrf(forwardPowerDbm, POWERWIDTH - 1, DEFAULTPRECISION, buffer);
          }  

          sprintf(lineOneBufferout,"  FWD: %s dBm ",buffer);

          lcd.home();
          lcd.print(lineOneBufferout);
  
          dtostrf(reflectedPowerDbm, POWERWIDTH - 1, DEFAULTPRECISION, buffer);
          sprintf(lineOneBufferout,"  REF: %s dBm ",buffer);      

          lcd.setCursor(0,1);
          lcd.print(lineOneBufferout);
      
          break;

      case 2:                           // Display No. 3: SWR with bar graph

          GOWHITE;
          dtostrf(swr, SWRWIDTH, DEFAULTPRECISION, buffer);
          sprintf(lineOneBufferout,"  SWR: %s:1   ",buffer);
          swrgraph = constrain( (int) (( swr ) * 100), 100, 580);
          swrgraph = map(swrgraph, 100, 580, 0, 490) + 10;  // Added the " +10" so that something is always visible

          lcd.home();
          lcd.print(lineOneBufferout);
          ClearBarGraph();
          lbg.drawValue(swrgraph, 500);                      // display bar graph of SWR
          break;

      case 3:                           // Display No. 4:  Forward and Reflected power in Watts plus SWR

if ( forwardPowerDbm > 40.0 ){
            dtostrf(forwardPower, POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"FWD%s W  SWR:",buffer);      
          }
          else if ( forwardPowerDbm > 30.0 ){
            dtostrf(forwardPower, POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"FWD%s W  SWR:",buffer);      
          }
          else if ( forwardPowerDbm > 20.0 ){
            dtostrf(forwardPowermW, POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"FWD%s mW SWR:",buffer);
          }    
          else if ( forwardPowerDbm > 10.0 ){
            dtostrf(forwardPowermW, POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"FWD%s mW SWR:",buffer);
          }
          else if ( forwardPowerDbm > 0.0 ){
            dtostrf(forwardPowermW, POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"FWD%s mW SWR:",buffer);
          }
          else {
            dtostrf(forwardPoweruW, POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"FWD%s uW SWR:",buffer);
          }

          lcd.home();
          lcd.print(lineOneBufferout);
  
          if ( reflectedPowerDbm > 40.0 ){
            dtostrf(reflectedPower, POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"REF%s W  ",buffer);
          }
          else if ( reflectedPowerDbm > 30.0 ){
            dtostrf(reflectedPower, POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"REF%s W   ",buffer);      
          }
          else if ( reflectedPowerDbm > 20.0 ){
            dtostrf(reflectedPowermW , POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"REF%s mW  ",buffer);
          }    
          else if ( reflectedPowerDbm > 10.0 ){
            dtostrf(reflectedPowermW , POWERWIDTH, DEFAULTPRECISION, buffer);
            sprintf(lineOneBufferout,"REF%s mW  ",buffer);
          }
          else if ( reflectedPowerDbm > 0.0 ){
            dtostrf(reflectedPowermW , POWERWIDTH, PRECISIONTWO, buffer);
            sprintf(lineOneBufferout,"REF%s mW  ",buffer);
          }
          else {
            dtostrf(reflectedPoweruW, POWERWIDTH, PRECISIONZERO, buffer);
            sprintf(lineOneBufferout,"REF%s uW  ",buffer);
          }

          lcd.setCursor(0,1);
          lcd.print(lineOneBufferout);
          
          dtostrf(swr, SWRWIDTH - 1, DEFAULTPRECISION, buffer);
          lcd.setCursor(13,1);
          lcd.print(buffer);
      
          break;
      }
    }
  }
  if (buttoncount == 4) {                   // reset BUTTONCOUNT so we can start from the beginning
    buttoncount = 0;
  }
}
// ===========================================================================================================
// ============================== End of loop() ==============================================================
// ===========================================================================================================

/* ===========================================================================================================
* 
*  Calculate the slope of the log amp output curve
*
*  Input parameters:
*     int X1    - analog value for sample #1
*     double Y1 - power for sample #1
*     int X2    - analog value for sample #2
*     double Y1 - power for sample #2
*
*  Return parameters:
*     float val - slope of the two sample points 
*
******/

float slope(int X1, double Y1, int X2, double Y2){  
  
float  val = ( X1 - X2 ) / (( 10.0 * log10( 1000.0 * Y1)) - (10.0 * log10( 1000.0 * Y2)));
  
  return val;
  
}

/* ===========================================================================================================
* 
*  Calculate the intercept of the log amp output curve
*
*  Input parameters:
*     double Y1 - power
*     int X1    - analog value
*     slope     - slope 
*
*  Return parameters:
*     float val - slope of the two sample points 
*
******/

float intercept(float Y1, int X1, float slope){  
  
  float  val = (10.0 * log10(1000.0 * Y1)) - ( X1 / slope);
  
  return val;
  
}

/* ===========================================================================================================
*
*   Convert dBm to Watts
*
*   Input parameters:
*       float dBm    power in dBm to be converted
*       int   order  order of magnitude ... Watts = 1, mW = 1000 or uW = 1000000
*
*   Return parameters
*        float power
*
*****/

float dBmToWatts(float dBm, long order){
  
  float power = (float) pow(10.0, (( dBm - 30.0) / 10.0 )) * order;
  
  return power;
  
}

/* ===========================================================================================================
*
*  Simple button debounce routine.  
*
*  Input parameters:
*    int buttonPin                          // where the switch is connected!
*    
*  Return parameters:
*    none
*
******/
  
void deBounce(int buttonPin)
{
  unsigned long now = millis ();
  do
  {
    if (digitalRead(buttonPin) == LOW)  // on bounce, reset time-out
      now = millis ();
  } while (digitalRead(buttonPin) == LOW || (millis() - now) <= DEBOUNCEDELAY);
}  
/* ===========================================================================================================
*
*  LCD Bar Graph does not clear any data from the row prior to writing to it.  Under normal circumstances 
*  an lcd.clear() would be sufficient.  However, we don't want to use lcd.clear on every write.  This
*  function executes an lcd.clear once following an alarm cleared or after a button push and branching
*  to display case 3 ... SWR plus SWR bar graph.
*
*  Input parameters:
*    void
*    
*  Return parameters:
*    none
*
******/
  
void ClearBarGraph()
{
  if ( OneShotOnly == true ){
    lcd.clear();
    OneShotOnly = false;
  }
}  
/* ===========================================================================================================
*
*  Alarm function -- updates deiplay to reflect alarm state
*
*  Input parameters
*    int alarmtype
*
*  Return values
*    none

*
*****/
void DisplayAlarm(int alarmtype)
{
   switch(alarmtype){
     case ALARM1:                              // Power exceeds 20 Watts
       lcd.home();
       lcd.print("  INPUT > 20 W  ");
       lcd.setCursor(0,1);
       lcd.print("  REDUCE POWER  ");
       OneShotOnly = true;
       break;

     case ALARM2:                              // Power greatly exceeds 20 Watts DANGER!!
       lcd.home();
       lcd.print(" INPUT >> 25 W  ");
       lcd.setCursor(0,1);
       lcd.print("REDUCE PWR NOW!!");
       OneShotOnly = true;
       break;
     case ALARM3:                              // Reflected power exceeds forward power
       lcd.home();
       lcd.print("     ERROR!     ");
       lcd.setCursor(0,1);
       lcd.print("REF EXCEEDS FOR ");
       break;
     case ALARM4:                              // SWR is increasing to a dangerous level
       lcd.home();
       lcd.print(lineOneBufferout);
       lcd.setCursor(0,1);
       lcd.print("  HIGH SWR!!!   ");
       break;
     case ALARM5:                              // SWR is increasing
       lcd.home();
       lcd.print("    HIGH SWR    ");
       lcd.setCursor(0,1);
       lcd.print("NEEDS ATTENTION!");
       break;
   }
}
/* ===========================================================================================================
*
*  Function used to flash backlight with timer1 interrupt.  Enabled by attaching interrupt:
*
*     timer1.attachInterrupt(FlashDisplay);
*
*  and it is disabled by detaching it:
*     
*     timer1.detachInterrupt();
*
*  Input parameters
*    void
*
*  Return values
*    none 
*
******/
void FlashDisplay()
{
  
  if (state != true) state = true;
  else state = false;

  switch(alarmType){
    case ALARM1:                          // Power exceeds 20 Watts
    case ALARM3:                          // Reflected power is greater than forward power
    case ALARM4:                          // Rising SWR Caution ... 
    {
      GOYELLOW;
      break;
    }
    case ALARM2:                          // Power greatly exceeds 20 Watts DANGER!!
    case ALARM5:                          // High SWR
    {
    if (state == true){
      GORED;
    }
    else GOBLACK;
      break;
    }
  }
}
