/*
  The heart of the code that manages the AD9850 was written by Richard Visokey AD7C - www.ad7c.com
  
 Modifications were made by Jack Purdum and Dennis Kidder:
   Rev 3.01:  Feb. 5, 2014
 
*/

// Include the library code
//#include <cmath.h>
#include <avr/pgmspace.h>
#include <LiquidCrystal.h>
#include <rotary.h>
#include <EEPROM.h>
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <DDSLCDShield.h>
#include "W6DQ_DDS_VFO.h"

//#define DEBUG 1

//Setup some items
#define VFOUPPERFREQUENCYLIMIT  30000000
#define VFOLOWERFREQUENCYLIMIT   1000000

#define CALIBRATIONOFFSET  -100  // this is used to adjust the output frequency to display frequency

#define W_CLK           8   // Pin 8 - connect to AD9850 module word load clock pin (CLK)
#define FQ_UD           9   // Pin 9 - connect to freq update pin (FQ)
#define DATA           10   // Pin 10 - connect to serial data load pin (DATA)
#define RESET          11   // Pin 11 - connect to reset pin (RST) 
#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }


#define TENHZ             0
#define FIFTYHZ           1
#define HUNDREDHZ         2
#define FIVEHUNDREDHZ     3
#define ONEKILOHZ         4
#define TWOANDAHALFKILOHZ 5
#define FIVEKILOHZ        6
#define TENKILOHZ         7
#define ONEHUNDREDKILOHZ  8
#define ONEMEGAHZ         9

#define LCDCOLS           16
#define INCREMENTSIZES    10      // Number of elements in the increment table.
#define MAXBANDS          9       // Number of ham bands covered
#define MAXUSERFREQS      10      // The maximum number of frequencies the user can store

#define FREQINBAND        0       // Used with range checking
#define FREQOUTOFBAND     1 

#define DELTAFREQOFFSET   25      // Must move frequency more than this to call it a frequency change
#define DELTATIMEOFFSET   10000   // If not change in frequency within 10 seconds, update the frequency
// ===================================== EEPROM Offsets ==============================================
#define READCURRENTFREQ    0                     // The EEPROM record address of the last written frequency
#define READUSERCOUNTS     (READCURRENTFREQ + 1) // The EEPROM record address of number of records saved by the user.
#define READLASTBANDFREQ   (READUSERCOUNTS + 1)  // The starting EEPROM record address for the 9 bands
#define READUSERSTOREDFREQ (READLASTBANDFREQ + 9)// The starting EEPROM record address for 10 user-saved frequencies. Note
                                 // that READUSERSTOREDFREQ plus the number stored at READUSERCOUNTS is the
                                 // next empty address.
#define READUPPERBANDEDGES READUSERSTOREDFREQ + 10  // The are the upper band edges for the 9 bands

#define ENCODERASLEEP     0       // The EEPROM encoder is just tuning
#define ENCODERREVIEW     1       // Scrollinig through user-stored EEPROM frequencies
#define ENCODERSTORE      2       // Ready to store a new frequency in user EEPROM

byte ones,tens,hundreds,thousands,tenthousands,hundredthousands,millions ;  // Placeholders


#define SPACES      "                "
#define HERTZ       "Hz"
#define KILOHERTZ   "kHz"
#define MEGAHERTZ   "MHz"
#define RECNO       "REC #:"
char temp[17];

int_fast32_t currentFrequency;                        // Starting frequency of VFO
int_fast32_t oldFrequency = 1;                        // variable to hold the updated frequency

int incrementIndex = 0;

static char* incrementStrings[] = {"10", "20", "100", ".5", "1", "2.5", "5", "10", "100", "1"};
static  long incrementTable[] = {10,20,100,500,1000,2500,5000,10000,100000,1000000};
static  long memory[] = {28000000, 24890000,21000000,18068000,14000000,10100000,7000000,3500000,1800000};

Rotary r = Rotary(2,3);             // sets the pins the rotary encoder uses.  Must be interrupt pins.
DDSLCDShield lcd = DDSLCDShield();
w6dqDDSVFO myVFO = w6dqDDSVFO();


//=================== EEPROM Data =====================

//int eepromUserRecordCount;            // The number of records the user has stored.
//int eepromWriteStatus;                // 0 if the frequency has not been save, 1 if it has
int eepromCurrentRecordIndex;         // The EEPROM record that is currently being used.
unsigned long currentEEPROMFrequency; // The EEPROM frequency being displayed
unsigned long markFrequency;          // The frequency just written to EEPROM
int EEPROMReadWrite;                  // 0 for reading, 1 for writing

long eepromStartTime;                 // Set when powered up and while tuning
long eepromCurrentTime;               // The current time reading
//unsigned long eepromStartFreq;

// ===================================================

void setup() {
#ifdef DEBUG
Serial.begin(115200);
#endif

// ===================== Set up EEPROM object ======================================
  EEPROMReadWrite = ENCODERASLEEP;                            // Assume no user EEPROM activity to start
  
  myVFO.currentFrequencyIncrement = incrementTable[0];        // Store working freq variables

  currentFrequency = readEEPROMRecord(READCURRENTFREQ);       // Last frequency read while tuning
  markFrequency = currentFrequency;
  eepromStartTime = millis();
  myVFO.currentIndex = FindBand(); 

  pinMode(A0,INPUT);       // Connect to a button that goes to GND on push
  digitalWrite(A0,HIGH);
  
  lcd.begin(16, 2);        // Display LCD
  
  PCICR |= (1 << PCIE2);                      // Interrupt service code
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();
  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 
  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.`  
}

void loop() {
 int flag;
 
  if (currentFrequency != oldFrequency){  // Are we still looking at the same frequency?
    NewShowFreq();                        // Nope, so update display.
    flag = DoRangeCheck();
    if (flag == FREQOUTOFBAND) {          // Tell user if out of band
      lcd.setCursor(0,0);
      lcd.print("*");
    }
    sendFrequency(currentFrequency);      // Send frequency to chip
    if (oldFrequency == 1) {
      int i;
      myVFO.getBandString(temp);
      if (myVFO.currentIndex == 8)         // Find column offset
        i = 12;
      else
        i = 13;
      lcd.setCursor(i,1);
      lcd.print(temp);      
    } 
    oldFrequency = currentFrequency;
  }

  eepromCurrentTime = millis();
  if (eepromCurrentTime - eepromStartTime > DELTATIMEOFFSET) {  // On this frequency long enough to store it?
    writeEEPROMRecord(currentFrequency, 0);
    //eepromStartFreq = currentFrequency;
    eepromStartTime = millis();
  }
  uint8_t buttons = lcd.readButtons();    // See if they pressed a button
  if (buttons) {
    ButtonPush(buttons);                  // Yep...
  }
}



// frequency calc from datasheet page 8 = <sys clock> * <frequency tuning word>/2^32
void sendFrequency(double frequency) {  
  int32_t freq = ( frequency + CALIBRATIONOFFSET) * 4294967295/125000000;  // note 125 MHz clock on 9850.  You can make 'slight' tuning 
                                                    // variations here by adjusting the clock frequency.
  for (int b = 0; b < 4; b++, freq >>= 8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for 9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}

// transfers a byte, a bit at a time, LSB first to the 9850 via serial DATA line
void tfr_byte(byte data)
{
  for (int i = 0; i < 8; i++, data >>= 1) {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}

/*****
  This method is used to change the frequency band. The button is passed in so we know whether the user
  is raising or lowering the band frequency. If the user increases beyond the freequency limits, the band 
  "rolls around" to the lowest band. Likewise, decreasing the band below the lowest frequency rolls around 
  to the highest frequency.
  
  Argument list:
    uint8_t button    Will be either BANDUPMODE or BANDDOWNMODE
  
  Return value:
    void
*****/
void ProcessBands(uint8_t button)
{
  //char temp[17];
 
  //DisplayLCDLine(spaces, 1, 0);                    // This clears the line
  DisplayLCDLine(SPACES, 1, 0);                    // This clears the line
  DisplayLCDLine(myVFO.getModeString(), 1, 0);
  myVFO.changeBand(button, temp);
  DisplayLCDLine(temp, 1, 12);
}

/*****
  This method is used to change the number of hertz associated with one click of the rotary encoder. The 
  button is passed in so we know whether the user   is raising or lowering the band frequency. Ten step 
  levels are provided and increasing or decreasing beyonds the array limits rolls around to the start/end
  of the array.
  
  Argument list:
    void
  
  Return value:
    void
*****/
void ProcessSteps()
{ 
  
  DisplayLCDLine(SPACES, 1, 0);                    // This clears the line
  DisplayLCDLine(myVFO.getModeString(), 1, 0);
    
  strcpy(temp, incrementStrings[incrementIndex]);

  if (incrementIndex < 3) {
    strcat(temp, HERTZ);
  } else {
    if (incrementIndex >= 3 && incrementIndex < INCREMENTSIZES - 1) {
      strcat(temp, KILOHERTZ);
    } else {
      strcat(temp, MEGAHERTZ);
    } 
  }
  DisplayLCDLine(temp, 1, LCDCOLS - strlen(temp)); 
}

/*****
  This method is used to read a record from EEPROM. Each record is 4 bytes (sizeof(unsigned long)) and
  is used to calculate where to read from EEPROM.
  
  Argument list:
    int record                the record to be read. While tuning, it is record 0
  
  Return value:
    unsigned long            the value of the record,
    
  CAUTION:  Record 0 is the current frequency while tuning, etc. Record 1 is the number of stored
            frequencies the user has set. Therefore, the stored frequencies list starts with record 23.
*****/
unsigned long readEEPROMRecord(int record)
{
  int offset;
  union {
    byte array[4];
    unsigned long val;
  } myUnion;
  
  offset = record * sizeof(unsigned long);

  myUnion.array[0] = EEPROM.read(offset);
  myUnion.array[1] = EEPROM.read(offset + 1);
  myUnion.array[2] = EEPROM.read(offset + 2);
  myUnion.array[3] = EEPROM.read(offset + 3);

  return myUnion.val;
}


/*****
  This method is used to test and perhaps write the latest frequency to EEPROM. This routine is called 
  every DELTATIMEOFFSET (default = 10 seconds). The method tests to see if the current frequency is the
  same as the last frequency (markFrequency). If they are the same, +/- DELTAFREQOFFSET (drift?), no
  write to EEPROM takes place. If the change was larger than DELTAFREQOFFSET, the new frequency is
  written to EEPROM. This is done because EEPROM can only be written/erased 100K times before it gets
  flaky.
  
  Argument list:
    unsigned long freq        the current frequency of VFO
    int record                the record to be written. While tuning, it is record 0
  
  Return value:
    void
*****/
void writeEEPROMRecord(unsigned long freq, int record)
{
  int offset;
  union {
    byte array[4];
    unsigned long val;
  } myUnion;
 
  if (abs(markFrequency - freq) < DELTAFREQOFFSET) {   // Is the new frequency more or less the same as the one last written?
    return;                                           // the same as the one last written? If so, go home.
  }
  myUnion.val = freq;
  offset = record * sizeof(unsigned long);

  EEPROM.write(offset, myUnion.array[0]);
  EEPROM.write(offset + 1, myUnion.array[1]);
  EEPROM.write(offset + 2, myUnion.array[2]);
  EEPROM.write(offset + 3, myUnion.array[3]);
  markFrequency = freq;                               // Save the value just written
}

/*****
  This method is used to find out which band the current EEPROM frequency is in.
  
  Argument list:
    void
  
  Return value:
    int            an index used to display the current band by setting myVFO.currentIndex
*****/
int FindBand()
{
  int i;
  
  for (i = 1; i < sizeof(memory)/sizeof(long); i++) {
    if (memory[i] - currentFrequency <= 0)
      break;
  }
  return i;
}


/*****
  This method is used to process the various button presses by the user.
  
  Argument list:
    uint8_t button    Will be either BANDUPMODE or BANDDOWNMODE
  
  Return value:
    void
*****/
void ButtonPush(uint8_t button)
{

  delay(BOUNCEDELAY);

  switch (button) {
    case FUNCTION:                                          // REVIEW and STORE 
      lcd.setCursor(0, 1);
      lcd.print(SPACES);
      if (EEPROMReadWrite == ENCODERASLEEP) {
        eepromCurrentRecordIndex = READUSERSTOREDFREQ - 1;   // Start of user-stored records, less 1 cause of auto-increment
        EEPROMReadWrite = ENCODERREVIEW;
        myVFO.currentModeIndex = REVIEWEEPROMMODE;
        currentFrequency = readEEPROMRecord(READUSERSTOREDFREQ);
        DisplayLCDLine(myVFO.getModeString(), 1, 0);
      } else {
        myVFO.currentModeIndex = STOREEEPROMMODE;
        DisplayLCDLine(myVFO.getModeString(), 1, 0);
          
        if (EEPROMReadWrite == ENCODERREVIEW) {               // We want to store a frequency
          EEPROMReadWrite = ENCODERSTORE;                   
          currentEEPROMFrequency = currentFrequency;        // This is the freq we want to store
        }
      }
      break;
      
    case BANDUP:                                          // Go up to next band
      EEPROMReadWrite = ENCODERASLEEP;
      myVFO.currentModeIndex = BANDUPMODE;
      ProcessBands(button);
      currentFrequency = memory[myVFO.currentIndex];
      //NewShowFreq();
      break;
      
    case BANDDOWN:                                        // Go down a band
      EEPROMReadWrite = ENCODERASLEEP;
      myVFO.currentModeIndex = BANDDOWNMODE;
      ProcessBands(button);
      currentFrequency = memory[myVFO.currentIndex];
      break;
      
    case INCREMENTUP:                                    // Adjust STEP upwards
      EEPROMReadWrite = ENCODERASLEEP;
      myVFO.currentModeIndex = INCREMENTUPMODE;
      if (incrementIndex < INCREMENTSIZES - 1) {    // Adjust the increment size
        incrementIndex++;
      } else {
        incrementIndex = 0;    // Wrap around to zero
      }
      myVFO.currentFrequencyIncrement = incrementTable[incrementIndex];      
      ProcessSteps();  
      break;
      
    case INCREMENTDOWN:                                  // Adjust STEP downwards
      EEPROMReadWrite = ENCODERASLEEP;
      myVFO.currentModeIndex = INCREMENTDOWNMODE;
      if (incrementIndex > 0) {
        incrementIndex--;
      } else {
        incrementIndex = INCREMENTSIZES - 1;    // Wrap around to zero
      }
      myVFO.currentFrequencyIncrement = incrementTable[incrementIndex];  
      ProcessSteps();
      break;

    case ENCODER:                                       // Prcess encoder control
     switch (EEPROMReadWrite) {
        case ENCODERASLEEP:                             // Just tuning
          break;
        case ENCODERREVIEW:                             // In REVIEW mode
          break;
        case ENCODERSTORE:                              // In STORE mode
          markFrequency = 0;
          writeEEPROMRecord(currentFrequency, eepromCurrentRecordIndex);
          EEPROMReadWrite = ENCODERASLEEP;
          eepromCurrentRecordIndex = READUSERSTOREDFREQ - 1; // Start of user-stored records, less 1 cause of auto-increment
          myVFO.currentModeIndex = BANDUPMODE;
          currentFrequency = readEEPROMRecord(READCURRENTFREQ);       // Last frequency read while tuning
          oldFrequency = 0;
          ButtonPush(BANDUPMODE);
          break;
          
        default:                            // Assume most harmless
          EEPROMReadWrite = ENCODERASLEEP; 
          break;
      }
      break;
      
    default:    // ERROR
      myVFO.currentModeIndex = ERRORMODE;
      myVFO.currentIndex = 5;
      break;
  }
}

// Original interrupt service routine, as modified by Jack
ISR(PCINT2_vect) {
  unsigned char result = r.process();
  
  switch (result) {
    case 0:                                                 // Nothing done...
      return;
    case DIR_CW:                                            // Turning Clockwise, going to higher frequencies
      if (EEPROMReadWrite == ENCODERASLEEP || EEPROMReadWrite == ENCODERSTORE) {   //  Tuning...
        currentFrequency += myVFO.currentFrequencyIncrement;
      } else {
        eepromCurrentRecordIndex++;                            // Increment...
        if ((eepromCurrentRecordIndex) >= MAXUSERFREQS + READUSERSTOREDFREQ)  // Too far?
            eepromCurrentRecordIndex = READUSERSTOREDFREQ;
        if (EEPROMReadWrite == ENCODERREVIEW) {               // Reviewing user EEPROM address
          currentFrequency = readEEPROMRecord(eepromCurrentRecordIndex); 
        } else {
          if (EEPROMReadWrite == ENCODERSTORE) {                // Writing user EEPROM address
            currentFrequency = readEEPROMRecord(eepromCurrentRecordIndex); 
          }
        }
      }
      break; 

    case DIR_CCW:                                    // Turning Counter-Clockwise, going to lower frequencies      
      if (EEPROMReadWrite == ENCODERASLEEP || EEPROMReadWrite == ENCODERSTORE) {   //  Tuning...
        currentFrequency -= myVFO.currentFrequencyIncrement;
      } else {
        eepromCurrentRecordIndex--;                                  // Decrement...
        if ((eepromCurrentRecordIndex) <= READUSERSTOREDFREQ - 1)
            eepromCurrentRecordIndex = MAXUSERFREQS + READUSERSTOREDFREQ - 1;  // Too far?
        if (EEPROMReadWrite == ENCODERREVIEW) {                  // Reviewing user EEPROM address
          currentFrequency = readEEPROMRecord(eepromCurrentRecordIndex);
        } else {
          if (EEPROMReadWrite == ENCODERSTORE) {
            currentFrequency = readEEPROMRecord(eepromCurrentRecordIndex); 
          }
        }
      }
      break;    
    default:
       // Should never be here
      break;
  }        
    if (currentFrequency >= VFOUPPERFREQUENCYLIMIT){
      currentFrequency = oldFrequency;
    } // UPPER VFO LIMIT
    if (currentFrequency <= VFOLOWERFREQUENCYLIMIT){
      currentFrequency = oldFrequency;
    } // LOWER VFO LIMIT  
}


/*****
  This method is used to format a frrequency on the lcd display. The currentFrequqncy variable holds the display 
  frequency.
  
  Argument list:
    void
  
  Return value:
    void
*****/
void NewShowFreq(){
  int offset = 0;
  
  millions = int(currentFrequency/1000000);
  if (millions > 9) {
    temp[0] = (char) (millions / 10) + 48;
    temp[1] = (char) (millions % 10) + 48;
    offset++;
  } else { 
    temp[0] = (char) (millions + 48);
  }
  temp[1 + offset] = ',';
  hundredthousands = ((currentFrequency/100000)%10);
  temp[2 + offset] = (char) (hundredthousands + 48);
  tenthousands = ((currentFrequency/10000)%10);
  temp[3 + offset] = (char) (tenthousands + 48);
  thousands = ((currentFrequency/1000)%10);
  temp[4 + offset] = (char) (thousands + 48);
  temp[5 + offset] = '.';
  hundreds = ((currentFrequency/100)%10);
  temp[6 + offset] = (char) (hundreds + 48);
  tens = ((currentFrequency/10)%10);
  temp[7 + offset] = (char) (tens + 48);
  ones = ((currentFrequency/1)%10);
  temp[8 + offset] = (char) (ones + 48);
  temp[9 + offset] = ' ';
  temp[10 + offset] = '\0';
  strcat(temp, KILOHERTZ);
  lcd.setCursor(0,0);
  lcd.print(SPACES);
  lcd.setCursor(2 - offset, 0);
  lcd.print(temp);
}
/*****
  This method is used to format a frrequency on the lcd display. The currentFrequqncy variable holds the display 
  frequency.
  
  Argument list:
    void
  
  Return value:
    void
*****/
void DisplayLCDLine(char *message, int row, int col)
{
  lcd.setCursor(col, row);
  lcd.print(message);
}

/*****
  This method is used to see if the current frequency displayed on line 1 is within a ham band.
  
  Argument list:
    void
  
  Return value:
    int            0 (FREQINBAND) if within a ham band, 1 (FREQOUTOFBAND) if not
*****/
int DoRangeCheck()
{
  int i;
  long lowDeviation, hiDeviation;
  unsigned long freqLow, freqHigh;
    
  for (i = 0; i < MAXBANDS; i++) {
    freqLow = readEEPROMRecord(READLASTBANDFREQ + i);      // Get lower band edge
    freqHigh = readEEPROMRecord(READUPPERBANDEDGES + i);   // Get upper band edge
    lowDeviation = freqLow - currentFrequency;
    hiDeviation = freqHigh - currentFrequency;
    if (freqLow <= currentFrequency && freqHigh >= currentFrequency) { // Within band range...OK
      return FREQINBAND;
    }
    if (currentFrequency > freqLow)  // Doesn't fall in a ham band
      return FREQOUTOFBAND;
  } 
}
  
